# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialogs for shifting positions."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("FrameShiftDialog", "TimeShiftDialog")


class PositionShiftDialog(gaupol.BuilderDialog):

    """Base class for dialogs for shifting positions."""

    _widgets = ("all_radio",
                "amount_spin",
                "current_radio",
                "preview_button",
                "selected_radio",
                "unit_label")

    def __init__(self, parent, application):
        """Initialize a :class:`PositionShiftDialog` object."""
        gaupol.BuilderDialog.__init__(self, "shift-dialog.ui")
        self.application = application
        self._init_widgets()
        self._init_values()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_OK)

    def _get_preview_row(self):
        """Return row to start preview from."""
        target = self._get_target()
        page = self.application.get_current_page()
        rows = page.view.get_selected_rows()
        return (rows[0] if rows else 0)

    def _get_target(self):
        """Return the selected target."""
        if self._selected_radio.get_active():
            return gaupol.targets.SELECTED
        if self._current_radio.get_active():
            return gaupol.targets.CURRENT
        if self._all_radio.get_active():
            return gaupol.targets.ALL
        raise ValueError("Invalid target radio state")

    def _init_values(self):
        """Intialize default values for widgets."""
        target = gaupol.conf.position_shift.target
        self._selected_radio.set_active(target == gaupol.targets.SELECTED)
        self._current_radio.set_active(target == gaupol.targets.CURRENT)
        self._all_radio.set_active(target == gaupol.targets.ALL)
        page = self.application.get_current_page()
        rows = page.view.get_selected_rows()
        if (not rows) and (target == gaupol.targets.SELECTED):
            self._current_radio.set_active(True)
        self._selected_radio.set_sensitive(bool(rows))
        if page.project.video_path is None:
            self._preview_button.set_sensitive(False)
        if page.project.main_file is None:
            self._preview_button.set_sensitive(False)
        self._amount_spin.emit("value-changed")

    def _init_widgets(self):
        """Initialize widgets."""
        raise NotImplementedError

    def _on_amount_spin_value_changed(self, spin_button):
        """Set response sensitivity."""
        has_value = (spin_button.get_value() != 0.0)
        self.set_response_sensitive(gtk.RESPONSE_OK, has_value)

    def _on_preview_button_clicked(self, *args):
        """Preview shift changes with a video player."""
        page = self.application.get_current_page()
        target = self._get_target()
        rows = self.application.get_target_rows(target)
        self.application.preview_changes(page,
                                         self._get_preview_row(),
                                         aeidon.documents.MAIN,
                                         page.project.shift_positions,
                                         (rows, self._get_amount()))

    def _on_response(self, dialog, response):
        """Save target and shift positions."""
        gaupol.conf.position_shift.target = self._get_target()
        if response == gtk.RESPONSE_OK:
            self._shift_positions()

    def _shift_positions(self):
        """Shift positions in subtitles."""
        gaupol.util.set_cursor_busy(self)
        target = self._get_target()
        rows = self.application.get_target_rows(target)
        amount = self._get_amount()
        for page in self.application.get_target_pages(target):
            page.project.shift_positions(rows, amount)
        gaupol.util.set_cursor_normal(self)


class FrameShiftDialog(PositionShiftDialog):

    """Dialog for shifting frames."""

    __metaclass__ = aeidon.Contractual

    def _get_amount_ensure(self, value):
        assert isinstance(value, int)

    def _get_amount(self):
        """Return the amount of frames to shift."""
        return self._amount_spin.get_value_as_int()

    def _init_widgets(self):
        """Initialize widgets."""
        self._amount_spin.set_numeric(True)
        self._amount_spin.set_digits(0)
        self._amount_spin.set_increments(1, 10)
        self._amount_spin.set_range(-9999999, 9999999)
        self._amount_spin.set_value(0)
        self._unit_label.set_text(_("frames"))


class TimeShiftDialog(PositionShiftDialog):

    """Dialog for shifting times."""

    __metaclass__ = aeidon.Contractual

    def _get_amount_ensure(self, value):
        assert isinstance(value, float)

    def _get_amount(self):
        """Return the amount of seconds to shift."""
        return self._amount_spin.get_value()

    def _init_widgets(self):
        """Initialize widgets."""
        self._amount_spin.set_numeric(True)
        self._amount_spin.set_digits(3)
        self._amount_spin.set_increments(0.1, 1)
        self._amount_spin.set_range(-99999, 99999)
        self._amount_spin.set_value(0.000)
        self._unit_label.set_text(_("seconds"))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class TestSearchAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.delegate = self.application._on_find_next_activate.im_self

    def test__on_search_dialog_response(self):
        self.application.get_action("find_and_replace").activate()
        self.delegate._search_dialog.response(gtk.RESPONSE_CLOSE)

    def test_on_find_and_replace_activate(self):
        self.application.get_action("find_and_replace").activate()
        self.application.get_action("find_and_replace").activate()

    def test_on_find_next_activate(self):
        self.application.get_action("find_and_replace").activate()
        self.delegate._search_dialog._pattern_entry.set_text("a")
        self.delegate._search_dialog.next()
        self.application.get_action("find_next").activate()
        self.delegate._search_dialog.response(gtk.RESPONSE_CLOSE)
        self.application.get_action("find_next").activate()

    def test_on_find_previous_activate(self):
        self.application.get_action("find_and_replace").activate()
        self.delegate._search_dialog._pattern_entry.set_text("a")
        self.delegate._search_dialog.previous()
        self.delegate._search_dialog.emit("delete-event", None)
        self.application.get_action("find_previous").activate()
        self.delegate._search_dialog.response(gtk.RESPONSE_CLOSE)
        self.application.get_action("find_previous").activate()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for subtitle field types."""

import aeidon
_ = aeidon.i18n._

__all__ = ("fields",)


class Number(aeidon.EnumerationItem):

    is_position = False
    is_text = False
    label = _("No.")
    tooltip = _("Subtitle number")


class Start(aeidon.EnumerationItem):

    is_position = True
    is_text = False
    label = _("Start")
    tooltip = _("Start position")


class End(aeidon.EnumerationItem):

    is_position = True
    is_text = False
    label = _("End")
    tooltip = _("End position")


class Duration(aeidon.EnumerationItem):

    is_position = True
    is_text = False
    # Translators: 'Dur.' is short for duration. It is used in the header of a
    # tree view column that contains numbers five characters wide.
    label = _("Dur.")
    tooltip = _("Duration")


class MainText(aeidon.EnumerationItem):

    is_position = False
    is_text = True
    label = _("Main Text")
    tooltip = _("Main text")


class TranslationText(aeidon.EnumerationItem):

    is_position = False
    is_text = True
    label = _("Translation Text")
    tooltip = _("Translation text")


fields = aeidon.Enumeration()
fields.NUMBER = Number()
fields.START = Start()
fields.END = End()
fields.DURATION = Duration()
fields.MAIN_TEXT = MainText()
fields.TRAN_TEXT = TranslationText()



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Dialog for checking and correcting spelling."""

import aeidon
import gaupol
import gtk
import os
import pango
import sys
_ = aeidon.i18n._

__all__ = ("SpellCheckDialog",)


class SpellCheckDialog(gaupol.BuilderDialog):

    """Dialog for checking and correcting spelling.

    :cvar _max_replacements: Maximum amount of replacements to save to file
    :cvar _personal_dir: Directory for user replacement files
    :ivar _checker: :class:`enchant.Checker` instance used
    :ivar _doc: :attr:`aeidon.documents` item currectly being checked
    :ivar _entry_handler: Handler for replacement entry's "changed" signal
    :ivar _language: Language code for :attr:`_checker`
    :ivar _new_rows: List of rows in the current page with changed texts
    :ivar _new_texts: List of changed texts in the current page
    :ivar _page: :class:`gaupol.Page` instance currectly being checked
    :ivar _pager: Iterator to iterate over all target pages
    :ivar _replacements: List of misspelled words and their replacements
    :ivar _row: Row currently being checked
    """

    __metaclass__ = aeidon.Contractual
    _max_replacements = 10000
    _personal_dir = os.path.join(aeidon.CONFIG_HOME_DIR, "spell-check")

    _widgets = ("add_button",
                "edit_button",
                "entry",
                "ignore_all_button",
                "ignore_button",
                "join_back_button",
                "join_forward_button",
                "language_label",
                "replace_all_button",
                "replace_button",
                "table",
                "text_view",
                "tree_view")

    def __init___require(self, parent, application):
        assert aeidon.util.enchant_available()

    def __init__(self, parent, application):
        """Initialize a :class:`SpellCheckDialog` object.

        Raise :exc:`ValueError` if dictionary initialization fails.
        """
        gaupol.BuilderDialog.__init__(self, "spellcheck-dialog.ui")
        self._checker = None
        self._doc = None
        self._entry_handler = None
        self._language = gaupol.conf.spell_check.language
        self._new_rows = []
        self._new_texts = []
        self._page = None
        self._pager = None
        self._replacements = []
        self._row = None
        self.application = application
        self._init_spell_check()
        self._init_widgets()
        self._init_sensitivities()
        self._dialog.set_transient_for(parent)
        self._dialog.set_default_response(gtk.RESPONSE_CLOSE)
        self._start()

    def _advance(self):
        """Advance to the next spelling error."""
        while True:
            try:
                # Advance to the next spelling error in the current text.
                return self._advance_current()
            except StopIteration:
                # Save the current text if changes were made.
                text = unicode(self._checker.get_text())
                subtitle = self._page.project.subtitles[self._row]
                if text != subtitle.get_text(self._doc):
                    self._new_rows.append(self._row)
                    self._new_texts.append(text)
            # Move to the next row in the current page, move to the next page
            # in the sequence of target pages or end when all pages checked.
            try: self._advance_row()
            except StopIteration:
                # pylint: disable=E1101
                try: self._pager.next()
                except StopIteration: break
        self._set_done()

    def _advance_current(self):
        """Advance to the next spelling error in the current text.

        Raise :exc:`StopIteration` when no more errors in the current text.
        """
        self._checker.next()
        col = self._page.document_to_text_column(self._doc)
        self._page.view.set_focus(self._row, col)
        self._page.view.scroll_to_row(self._row)
        text = unicode(self._checker.get_text())
        text_buffer = self._text_view.get_buffer()
        text_buffer.set_text(text)
        a = self._checker.wordpos
        z = a + len(self._checker.word)
        start = text_buffer.get_iter_at_offset(a)
        end = text_buffer.get_iter_at_offset(z)
        text_buffer.apply_tag_by_name("misspelled", start, end)
        mark = text_buffer.create_mark(None, end, True)
        self._text_view.scroll_to_mark(mark, 0.2)
        leading = self._checker.leading_context(1)
        trailing = self._checker.trailing_context(1)
        self._join_back_button.set_sensitive(leading.isspace())
        self._join_forward_button.set_sensitive(trailing.isspace())
        self._set_entry_text("")
        self._populate_tree_view(self._checker.suggest())
        self._tree_view.grab_focus()

    def _advance_row(self):
        """Advance to the next subtitle and feed its text to the spell-checker.

        Raise :exc:`StopIteration` when no more subtitles in the current page.
        """
        self._row += 1
        if self._row >= len(self._page.project.subtitles):
            raise StopIteration
        subtitle = self._page.project.subtitles[self._row]
        text = subtitle.get_text(self._doc)
        self._checker.set_text(unicode(text))

    def _get_next_page(self):
        """Return the next page to check spelling in."""
        field = gaupol.conf.spell_check.field
        doc = gaupol.util.text_field_to_document(field)
        target = gaupol.conf.spell_check.target
        for page in self.application.get_target_pages(target):
            self.application.set_current_page(page)
            self._page = page
            self._doc = doc
            self._row = -1
            self._advance_row()
            yield page
            self._register_changes()

    def _init_checker(self):
        """Initialize spell-checker and its dictionary.

        Raise :exc:`ValueError` if dictionary initialization fails.
        """
        import enchant.checker
        try:
            dictionary = enchant.Dict(self._language)
            # Sometimes enchant will initialize a dictionary that will not
            # actually work when trying to use it, hence check something.
            dictionary.check("aeidon")
        except enchant.Error as (message,):
            self._show_error_dialog(message)
            raise ValueError("Dictionary initialization failed "
                             "for language %s" % repr(self._language))

        self._checker = enchant.checker.SpellChecker(dictionary, "")

    def _init_replacements(self):
        """Read misspelled words and their replacements from file."""
        basename = "%s.repl" % gaupol.conf.spell_check.language
        path = os.path.join(self._personal_dir, basename)
        if not os.path.isfile(path): return
        try: lines = aeidon.util.readlines(path)
        except (IOError, OSError): lines = []
        for line in aeidon.util.get_unique(lines):
            misspelled, correct  = line.strip().split("|", 1)
            self._replacements.append((misspelled, correct))

    def _init_sensitivities(self):
        """Initialize widget sensitivities."""
        self._join_back_button.set_sensitive(False)
        self._join_forward_button.set_sensitive(False)
        self._replace_all_button.set_sensitive(False)
        self._replace_button.set_sensitive(False)

    def _init_spell_check(self):
        """Initialize spell-check components and related widgets.

        Raise :exc:`ValueError` if dictionary initialization fails.
        """
        aeidon.util.makedirs(self._personal_dir)
        self._init_checker()
        self._init_replacements()
        try: name = aeidon.locales.code_to_name(self._language)
        except LookupError: name = self._language
        self._language_label.set_markup("<b>%s</b>" % name)

    def _init_tree_view(self):
        """Initialize the suggestion tree view."""
        selection = self._tree_view.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.connect("changed", self._on_tree_view_selection_changed)
        store = gtk.ListStore(str)
        self._tree_view.set_model(store)
        column = gtk.TreeViewColumn("", gtk.CellRendererText(), text=0)
        self._tree_view.append_column(column)

    def _init_widgets(self):
        """Initialize widget properties."""
        self._init_tree_view()
        font = gaupol.util.get_font()
        gaupol.util.set_widget_font(self._entry, font)
        gaupol.util.set_widget_font(self._text_view, font)
        gaupol.util.set_widget_font(self._tree_view, font)
        text_buffer = self._text_view.get_buffer()
        text_buffer.create_tag("misspelled", weight=pango.WEIGHT_BOLD)
        gaupol.util.scale_to_size(self._text_view, 56, 5, font)
        gaupol.util.scale_to_size(self._tree_view, 20, 9, font)
        self._entry_handler = self._entry.connect("changed",
                                                  self._on_entry_changed)

    def _on_add_button_clicked(self, *args):
        """Add the current word to the user dictionary."""
        self._checker.dict.add(unicode(self._checker.word))
        self._advance()

    def _on_edit_button_clicked(self, *args):
        """Edit the current text in a separate dialog."""
        text = unicode(self._checker.get_text())
        dialog = gaupol.TextEditDialog(self._dialog, text)
        response = gaupol.util.run_dialog(dialog)
        text = dialog.get_text()
        dialog.destroy()
        if response != gtk.RESPONSE_OK: return
        self._checker.set_text(unicode(text))
        self._advance()

    def _on_entry_changed(self, entry):
        """Populate suggestions based on text in `entry`."""
        word = unicode(entry.get_text())
        suggestions = self._checker.suggest(word)
        self._populate_tree_view(suggestions, False)
        self._replace_button.set_sensitive(bool(word))
        self._replace_all_button.set_sensitive(bool(word))

    def _on_ignore_all_button_clicked(self, *args):
        """Ignore all instances of the current word."""
        self._checker.ignore_always()
        self._advance()

    def _on_ignore_button_clicked(self, *args):
        """Ignore the current word."""
        self._advance()

    def _on_join_back_button_clicked(self, *args):
        """Join the current word with the preceding word."""
        text = unicode(self._checker.get_text())
        a = self._checker.wordpos
        text = text[:a - 1] + text[a:]
        self._checker.set_text(text)
        self._advance()

    def _on_join_forward_button_clicked(self, *args):
        """Join the current word with the following word."""
        text = unicode(self._checker.get_text())
        z = self._checker.wordpos + len(self._checker.word)
        text = text[:z] + text [z + 1:]
        self._checker.set_text(text)
        self._advance()

    def _on_replace_all_button_clicked(self, *args):
        """Replace all instances of the current word."""
        misspelled = unicode(self._checker.word)
        correct = self._entry.get_text()
        self._replacements.append((misspelled, correct))
        self._checker.replace_always(unicode(correct))
        self._advance()

    def _on_replace_button_clicked(self, *args):
        """Replace the current word."""
        misspelled = unicode(self._checker.word)
        correct = self._entry.get_text()
        self._replacements.append((misspelled, correct))
        self._checker.replace(unicode(correct))
        self._advance()

    def _on_response(self, dialog, response):
        """Apply changes to the current page."""
        self._register_changes()
        self._set_done()

    def _on_tree_view_selection_changed(self, *args):
        """Copy the selected suggestion into the entry."""
        selection = self._tree_view.get_selection()
        store, itr = selection.get_selected()
        if itr is None: return
        row = store.get_path(itr)[0]
        self._set_entry_text(store[row][0])

    def _populate_tree_view(self, suggestions, select=True):
        """Populate the tree view with `suggestions`."""
        word = unicode(self._checker.word)
        replacements = [x[1] for x in reversed(self._replacements)
                        if x[0] == word]

        suggestions = list(replacements) + list(suggestions)
        suggestions = aeidon.util.get_unique(suggestions)
        store = self._tree_view.get_model()
        store.clear()
        for suggestion in suggestions:
            store.append((suggestion,))
        if select and (len(store) > 0):
            self._tree_view.set_cursor(0)
            self._tree_view.scroll_to_cell(0)

    def _register_changes(self):
        """Register changes to the current page."""
        if not self._new_rows: return
        self._page.project.replace_texts(self._new_rows,
                                         self._doc,
                                         self._new_texts)

        self._page.project.set_action_description(
            aeidon.registers.DO, _("Spell-checking"))

        self._new_rows = []
        self._new_texts = []

    def _set_done(self):
        """Set state of widgets for finished spell-check."""
        self._text_view.get_buffer().set_text("")
        self._set_entry_text("")
        self._populate_tree_view(())
        self._table.set_sensitive(False)
        self._write_replacements()

    def _set_entry_text(self, word):
        """Set `word` to the entry with its "changed" handler blocked."""
        self._entry.handler_block(self._entry_handler)
        self._entry.set_text(word)
        self._entry.handler_unblock(self._entry_handler)
        self._replace_button.set_sensitive(bool(word))
        self._replace_all_button.set_sensitive(bool(word))

    def _show_error_dialog(self, message):
        """Show an error dialog after failing to load dictionary."""
        try: name = aeidon.locales.code_to_name(self._language)
        except LookupError: name = self._language
        title = _('Failed to load dictionary for language "%s"') % name
        dialog = gaupol.ErrorDialog(self._dialog, title, message)
        dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        gaupol.util.flash_dialog(dialog)

    def _start(self):
        """Start checking the spelling."""
        self._pager = self._get_next_page()
        self._pager.next()
        self._advance()

    def _write_replacements(self):
        """Write misspelled words and their replacements to file."""
        if not self._replacements: return
        self._replacements = aeidon.util.get_unique(self._replacements, True)
        basename = "%s.repl" % self._language
        path = os.path.join(self._personal_dir, basename)
        if len(self._replacements) > self._max_replacements:
            # Discard the oldest of replacements.
            self._replacements[-self._max_replacements:]
        text = os.linesep.join("|".join(x) for x in self._replacements)
        try: aeidon.util.write(path, text + os.linesep)
        except (IOError, UnicodeError):
            aeidon.util.print_write_io(sys.exc_info(), path)



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Model for subtitle data."""

import aeidon

__all__ = ("Project",)


class ProjectMeta(aeidon.Contractual):

    """Project metaclass with delegated methods added.

    Public methods are added to the class dictionary during :meth:`__new__`
    in order to fool Sphinx (and perhaps other API documentation generators)
    into thinking that the resulting instantiated class actually contains those
    methods, which it does not since the methods are be removed during
    :meth:`Project.__init__`.
    """

    def __new__(meta, class_name, bases, dic):
        new_dict = dic.copy()
        for agent_class_name in aeidon.agents.__all__:
            agent_class = getattr(aeidon.agents, agent_class_name)
            def is_public_method(name):
                return (not name.startswith("_") and
                        callable(getattr(agent_class, name)))

            attr_names = filter(is_public_method, dir(agent_class))
            for attr_name in attr_names:
                new_dict[attr_name] = getattr(agent_class, attr_name)
        return type.__new__(meta, class_name, bases, new_dict)


class Project(aeidon.Observable):

    """Model for subtitle data.

    :ivar _delegations: Dictionary mapping method names to agent methods
    :ivar calc: Instance of :class:`aeidon.Calculator` used
    :ivar clipboard: Instance of :class:`aeidon.Clipboard` used
    :ivar framerate: :attr:`aeidon.framerates` item corresponding to video
    :ivar main_changed: Integer, status of main document

       At unchanged state (i.e. file on disk corresponds to the state of the
       document) this value is zero. Doing and redoing increase the value by
       one  and undoing decreases value by one.

    :ivar main_file: Main instance of :class:`aeidon.SubtitleFile`
    :ivar redoables: Stack of :class:`aeidon.RevertableAction` instances
    :ivar subtitles: List of :class:`aeidon.Subtitle` instances
    :ivar tran_changed: Integer, status of translation document

       At unchanged state (i.e. file on disk corresponds to the state of the
       document) this value is zero. Doing and redoing increase the value by
       one  and undoing decreases value by one.

    :ivar tran_file: Translation instance of :class:`aeidon.SubtitleFile`
    :ivar undo_limit: Maximum size of undo/redo stacks or None for no limit
    :ivar undoables: Stack of :class:`aeidon.RevertableAction` instances
    :ivar video_path: Full, absolute path to the video file on disk

    Signals and their arguments for callback functions:
     * ``action-done``: project, action
     * ``action-redone``: project, action
     * ``action-undone``: project, action
     * ``main-file-opened``: project, main_file
     * ``main-file-saved``: project, main_file
     * ``main-texts-changed``: project, indices
     * ``positions-changed``: project, indices
     * ``subtitles-changed``: project, indices
     * ``subtitles-inserted``: project, indices
     * ``subtitles-removed``: project, indices
     * ``translation-file-opened``: project, tran_file
     * ``translation-file-saved``: project, tran_file
     * ``translation-texts-changed``: project, indices
    """

    __metaclass__ = ProjectMeta

    signals = ("action-done",
               "action-redone",
               "action-undone",
               "main-file-opened",
               "main-file-saved",
               "main-texts-changed",
               "positions-changed",
               "subtitles-inserted",
               "subtitles-removed",
               "subtitles-changed",
               "translation-file-opened",
               "translation-file-saved",
               "translation-texts-changed",)

    def __getattr__(self, name):
        """Return method delegated to an agent."""
        return self._delegations[name]

    def __init__(self, framerate=None, undo_limit=None):
        """Initialize a :class:`Project` object."""
        aeidon.Observable.__init__(self)
        framerate = framerate or aeidon.framerates.FPS_23_976
        self._delegations = {}
        self.calc = aeidon.Calculator(framerate)
        self.clipboard = aeidon.Clipboard()
        self.framerate = framerate
        self.main_changed = 0
        self.main_file = None
        self.redoables = []
        self.subtitles = []
        self.tran_changed = None
        self.tran_file = None
        self.undo_limit = undo_limit
        self.undoables = []
        self.video_path = None
        self._init_delegations()

    def _init_delegations(self):
        """Initialize the delegation mappings."""
        for agent_class_name in aeidon.agents.__all__:
            agent = getattr(aeidon.agents, agent_class_name)(self)
            def is_delegate_method(name):
                value = getattr(agent, name)
                return (callable(value) and
                        hasattr(value, "export") and
                        value.export is True)

            attr_names = filter(is_delegate_method, dir(agent))
            for attr_name in attr_names:
                attr_value = getattr(agent, attr_name)
                if attr_name in self._delegations:
                    raise ValueError("Multiple definitions of %s"
                                     % repr(attr_name))

                self._delegations[attr_name] = attr_value
                # Remove class-level function added by ProjectMeta.
                if hasattr(self.__class__, attr_name):
                    delattr(self.__class__, attr_name)



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSubtitle(aeidon.TestCase):

    def setup_method(self, method):
        self.tsub = aeidon.Subtitle()
        self.tsub.mode = aeidon.modes.TIME
        self.tsub.framerate = aeidon.framerates.FPS_25_000
        self.tsub.start = "00:00:01.000"
        self.tsub.end = "00:00:03.000"
        self.tsub.main_text = "main"
        self.tsub.tran_text = "translation"

        self.fsub = aeidon.Subtitle()
        self.fsub.mode = aeidon.modes.FRAME
        self.fsub.framerate = aeidon.framerates.FPS_25_000
        self.fsub.start = 100
        self.fsub.end = 300
        self.fsub.main_text = "main"
        self.fsub.tran_text = "translation"

    def test___cmp__(self):
        assert self.tsub <  self.fsub
        assert self.fsub >  self.tsub
        assert self.tsub == self.tsub
        assert self.fsub == self.fsub

    def test___cmp____value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda x: x < x, self.tsub)

    def test___getattr__(self):
        assert "ssa" not in dir(self.tsub)
        self.tsub.ssa.style = "test"
        assert "ssa" in dir(self.tsub)

    def test__get_duration(self):
        assert self.tsub.duration == "00:00:02.000"
        assert self.fsub.duration == 200

    def test__get_duration__value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda: self.tsub.duration)

    def test__get_duration_frame(self):
        assert self.tsub.duration_frame == 50
        assert self.fsub.duration_frame == 200

    def test__get_duration_seconds(self):
        assert self.tsub.duration_seconds == 2.0
        assert self.fsub.duration_seconds == 8.0

    def test__get_duration_time(self):
        assert self.tsub.duration_time == "00:00:02.000"
        assert self.fsub.duration_time == "00:00:08.000"

    def test__get_end(self):
        assert self.tsub.end == "00:00:03.000"
        assert self.fsub.end == 300

    def test__get_end_frame(self):
        assert self.tsub.end_frame == 75
        assert self.fsub.end_frame == 300

    def test__get_end_frame__value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda: self.tsub.end_frame)

    def test__get_end_seconds(self):
        assert self.tsub.end_seconds == 3.0
        assert self.fsub.end_seconds == 12.0

    def test__get_end_time(self):
        assert self.tsub.end_time == "00:00:03.000"
        assert self.fsub.end_time == "00:00:12.000"

    def test__get_end_time__value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda: self.tsub.end_time)

    def test__get_framerate(self):
        assert self.tsub.framerate == aeidon.framerates.FPS_25_000
        assert self.fsub.framerate == aeidon.framerates.FPS_25_000

    def test__get_main_text(self):
        assert self.tsub.main_text == u"main"
        assert self.fsub.main_text == u"main"

    def test__get_mode(self):
        assert self.tsub.mode == aeidon.modes.TIME
        assert self.fsub.mode == aeidon.modes.FRAME

    def test__get_start(self):
        assert self.tsub.start == "00:00:01.000"
        assert self.fsub.start == 100

    def test__get_start_frame(self):
        assert self.tsub.start_frame == 25
        assert self.fsub.start_frame == 100

    def test__get_start_frame__value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda: self.tsub.start_frame)

    def test__get_start_seconds(self):
        assert self.tsub.start_seconds == 1.0
        assert self.fsub.start_seconds == 4.0

    def test__get_start_time(self):
        assert self.tsub.start_time == "00:00:01.000"
        assert self.fsub.start_time == "00:00:04.000"

    def test__get_start_time__value_error(self):
        self.tsub._mode = None
        self.raises(ValueError, lambda: self.tsub.start_time)

    def test__get_tran_text(self):
        assert self.tsub.tran_text == u"translation"
        assert self.fsub.tran_text == u"translation"

    def test__set_duration__time(self):
        self.tsub.duration = "00:00:10.000"
        assert self.tsub.end_time == "00:00:11.000"
        self.tsub.duration = 500
        assert self.tsub.end_time == "00:00:21.000"
        self.tsub.duration = 10.0
        assert self.tsub.end_time == "00:00:11.000"

    def test__set_duration__frame(self):
        self.fsub.duration = 500
        assert self.fsub.end_frame == 600
        self.fsub.duration = "00:00:10.000"
        assert self.fsub.end_frame == 350
        self.fsub.duration = 5.0
        assert self.fsub.end_frame == 225

    def test__set_end__value_error(self):
        def function(x): self.tsub.end = x
        self.raises(ValueError, function, ())

    def test__set_end__time(self):
        self.tsub.end = "00:00:10.000"
        assert self.tsub.end_time == "00:00:10.000"
        self.tsub.end = 500
        assert self.tsub.end_time == "00:00:20.000"
        self.tsub.end = 5.0
        assert self.tsub.end_time == "00:00:05.000"

    def test__set_end__frame(self):
        self.fsub.end = 500
        assert self.fsub.end_frame == 500
        self.fsub.end = "00:00:10.000"
        assert self.fsub.end_frame == 250
        self.fsub.end = 5.0
        assert self.fsub.end_frame == 125

    def test__set_framerate(self):
        FPS_29_970 = aeidon.framerates.FPS_29_970
        self.tsub.framerate = FPS_29_970
        assert self.tsub.framerate == FPS_29_970

    def test__set_main_text(self):
        self.tsub.main_text = "test"
        assert self.tsub.main_text == u"test"

    def test__set_mode__time(self):
        self.tsub.mode = aeidon.modes.TIME
        self.tsub.mode = aeidon.modes.FRAME
        assert self.tsub._start == 25
        assert self.tsub._end == 75

    def test__set_mode__frame(self):
        self.fsub.mode = aeidon.modes.FRAME
        self.fsub.mode = aeidon.modes.TIME
        assert self.fsub._start == "00:00:04.000"
        assert self.fsub._end == "00:00:12.000"

    def test__set_start__time(self):
        self.tsub.start = "00:00:10.000"
        assert self.tsub.start_time == "00:00:10.000"
        self.tsub.start = 500
        assert self.tsub.start_time == "00:00:20.000"
        self.tsub.start = 5.0
        assert self.tsub.start_time == "00:00:05.000"

    def test__set_start__frame(self):
        self.fsub.start = 500
        assert self.fsub.start_frame == 500
        self.fsub.start = "00:00:10.000"
        assert self.fsub.start_frame == 250
        self.fsub.start = 5.0
        assert self.fsub.start_frame == 125

    def test__set_tran_text(self):
        self.tsub.tran_text = "test"
        assert self.tsub.tran_text == u"test"

    def test_convert_framerate__time(self):
        self.tsub.start = "00:00:01.000"
        self.tsub.end = "00:00:02.000"
        framerate = aeidon.framerates.FPS_23_976
        self.tsub.convert_framerate(framerate)
        assert self.tsub.framerate == framerate
        assert self.tsub.start == "00:00:01.043"
        assert self.tsub.end == "00:00:02.085"

    def test_convert_framerate__frame(self):
        self.fsub.start = 100
        self.fsub.end = 200
        framerate = aeidon.framerates.FPS_23_976
        self.fsub.convert_framerate(framerate)
        assert self.fsub.framerate == framerate
        assert self.fsub.start == 96
        assert self.fsub.end == 192

    def test_copy(self):
        copy = self.tsub.copy()
        assert copy == self.tsub
        assert copy is not self.tsub
        assert "ssa" not in dir(copy)
        self.tsub.ssa.style = "test"
        copy = self.tsub.copy()
        assert copy.ssa.style == "test"
        names = dir(self.tsub.ssa)
        for name in (x for x in names if not x.startswith("_")):
            value = getattr(self.tsub.ssa, name)
            assert getattr(copy.ssa, name) == value
        assert copy.ssa is not self.tsub.ssa

    def test_get_duration(self):
        TIME = aeidon.modes.TIME
        FRAME = aeidon.modes.FRAME
        assert self.tsub.get_duration(TIME) == "00:00:02.000"
        assert self.tsub.get_duration(FRAME) == 50
        assert self.fsub.get_duration(TIME) == "00:00:08.000"
        assert self.fsub.get_duration(FRAME) == 200

    def test_get_duration__value_error(self):
        self.raises(ValueError, self.tsub.get_duration, None)

    def test_get_end(self):
        TIME = aeidon.modes.TIME
        FRAME = aeidon.modes.FRAME
        assert self.tsub.get_end(TIME) == "00:00:03.000"
        assert self.tsub.get_end(FRAME) == 75
        assert self.fsub.get_end(TIME) == "00:00:12.000"
        assert self.fsub.get_end(FRAME) == 300

    def test_get_end__value_error(self):
        self.raises(ValueError, self.tsub.get_end, None)

    def test_get_start(self):
        TIME = aeidon.modes.TIME
        FRAME = aeidon.modes.FRAME
        assert self.tsub.get_start(TIME) == "00:00:01.000"
        assert self.tsub.get_start(FRAME) == 25
        assert self.fsub.get_start(TIME) == "00:00:04.000"
        assert self.fsub.get_start(FRAME) == 100

    def test_get_start__value_error(self):
        self.raises(ValueError, self.tsub.get_start, None)

    def test_get_text(self):
        MAIN = aeidon.documents.MAIN
        TRAN = aeidon.documents.TRAN
        assert self.tsub.get_text(MAIN) == "main"
        assert self.tsub.get_text(TRAN) == "translation"

    def test_get_text__value_error(self):
        self.raises(ValueError, self.tsub.get_text, None)

    def test_has_container(self):
        assert not self.tsub.has_container("subrip")
        self.tsub.subrip.x1 = 100
        assert self.tsub.has_container("subrip")

    def test_set_text(self):
        MAIN = aeidon.documents.MAIN
        TRAN = aeidon.documents.TRAN
        self.tsub.set_text(MAIN, "")
        assert self.tsub.main_text == ""
        self.tsub.set_text(TRAN, "")
        assert self.tsub.tran_text == ""

    def test_scale_positions__frame(self):
        self.fsub.scale_positions(2.0)
        assert self.fsub.start == 200
        assert self.fsub.end == 600

    def test_scale_positions__time(self):
        self.tsub.scale_positions(2.0)
        assert self.tsub.start == "00:00:02.000"
        assert self.tsub.end == "00:00:06.000"

    def test_shift_positions__frame(self):
        self.fsub.shift_positions(-10)
        assert self.fsub._start == 90
        assert self.fsub._end == 290

    def test_shift_positions__seconds(self):
        self.tsub.shift_positions(1.0)
        assert self.tsub._start == "00:00:02.000"
        assert self.tsub._end == "00:00:04.000"

    def test_shift_positions__time(self):
        self.tsub.shift_positions("00:00:01.000")
        assert self.tsub._start == "00:00:02.000"
        assert self.tsub._end == "00:00:04.000"



# Copyright (C) 2005-2008 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gtk
import gaupol


class TestTextEditDialog(gaupol.TestCase):

    def run__dialog__monospace(self):
        # pylint: disable=W0201
        gaupol.conf.editor.custom_font = "monospace"
        gaupol.conf.editor.use_custom_font = True
        self.dialog.destroy()
        self.dialog = gaupol.TextEditDialog(gtk.Window(), self.text)
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__sans(self):
        # pylint: disable=W0201
        gaupol.conf.editor.custom_font = "sans"
        gaupol.conf.editor.use_custom_font = True
        self.dialog.destroy()
        self.dialog = gaupol.TextEditDialog(gtk.Window(), self.text)
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        gaupol.conf.editor.length_unit = gaupol.length_units.CHAR
        self.text = "etaoin shrdlu etaoin shrdlu etaoin shrdlu etaoin shrdlu"
        self.dialog = gaupol.TextEditDialog(gtk.Window(), self.text)
        self.dialog.show()

    def test_get_text(self):
        self.dialog.set_text("test")
        text = self.dialog.get_text()
        assert text == "test"

    def test_set_text(self):
        self.dialog.set_text("test")
        text = self.dialog.get_text()
        assert text == "test"



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Reading and parsing data from subtitle files."""

import aeidon
import bisect


class OpenAgent(aeidon.Delegate):

    """Reading and parsing data from subtitle files."""

    # pylint: disable=E1101,W0201

    __metaclass__ = aeidon.Contractual

    def _align_translations_by_number(self, subtitles):
        """Add translation texts by aligning subtitle numbers."""
        indices = range(len(self.subtitles), len(subtitles))
        self.insert_subtitles(indices, register=None)
        for i, subtitle in enumerate(subtitles):
            self.subtitles[i].tran_text = subtitle.main_text

    def _align_translations_by_position(self, subtitles):
        """Add translation texts by aligning subtitle positions."""
        subtitles = [x.copy() for x in subtitles]
        mode = self.main_file.mode
        i = 0
        while subtitles:
            # Examine subtitles to be added one-by-one by comparing
            # their temporal middle positions with the start and end
            # positions of existing subtitles.
            ts = subtitles[0].get_start(mode)
            te = subtitles[0].get_end(mode)
            tm = self.calc.get_middle(ts, te)
            while True:
                # Skip over existing subtitles when
                # no suitable match found among translations.
                if i == len(self.subtitles): break
                ms = self.subtitles[i].get_start(mode)
                me = self.subtitles[i].get_end(mode)
                tm_cmp_ms = self.calc.compare(tm, ms)
                tm_cmp_me = self.calc.compare(tm, me)
                if tm_cmp_me <= 0: break
                i += 1
            if (i == len(self.subtitles)) or (tm_cmp_ms == -1):
                # Add a new subtitle when no suitable match
                # found among existing subtitles.
                subtitle = self.new_subtitle()
                subtitle.start = subtitles[0].start
                subtitle.end = subtitles[0].end
                self.subtitles.insert(i, subtitle)
                continue
            self.subtitles[i].tran_text = subtitles[0].main_text
            subtitles.pop(0)
            i += 1

    def _read_file(self, sfile):
        """Read `sfile` and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        Raise :exc:`aeidon.ParseError` if parsing fails.
        """
        try:
            return sfile.read()
        except (IOError, UnicodeError):
            raise
        except Exception:
            if not aeidon.debug:
                raise aeidon.ParseError("Failed to parse sfile %s"
                                        % repr(sfile))

            raise

    def _sort_subtitles_ensure(self, value, subtitles):
        sorted_subtitles, wrong_order_count = value
        for i in range(len(sorted_subtitles) - 1):
            assert sorted_subtitles[i] <= sorted_subtitles[i + 1]

    def _sort_subtitles(self, subtitles):
        """Sort and return `subtitles` and sort count.

        `subtitles` are sorted according to their start positions. Sort count
        is the amount of subtitles that needed to be moved in order to arrange
        them in ascending chronological order.
        """
        wrong_order_count = 0
        sorted_subtitles = []
        while subtitles:
            subtitle = subtitles.pop(0)
            index = bisect.bisect(sorted_subtitles, subtitle)
            if index < len(sorted_subtitles):
                wrong_order_count += 1
            sorted_subtitles.insert(index, subtitle)
        return sorted_subtitles, wrong_order_count

    @aeidon.deco.export
    def open(self, doc, path, encoding=None, *args, **kwargs):
        """Read and parse subtitle data for `doc` from `path`.

        `encoding` can be ``None`` to use the system default encoding.
        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        Raise :exc:`aeidon.FormatError` if unable to detect format.
        Raise :exc:`aeidon.ParseError` if parsing fails.

        Return the amount of subtitles that needed to be moved in order to
        arrange them in ascending chronological order.
        """
        if doc == aeidon.documents.MAIN:
            return self.open_main(path, encoding, *args, **kwargs)
        if doc == aeidon.documents.TRAN:
            return self.open_translation(path, encoding, *args, **kwargs)
        raise ValueError("Invalid document: %s" % repr(doc))

    def open_main_require(self, path, encoding):
        assert aeidon.encodings.is_valid_code(encoding)

    def open_main_ensure(self, value, path, encoding):
        assert self.main_file is not None
        assert self.main_changed == 0
        assert self.tran_file is None
        assert self.tran_changed is None

    @aeidon.deco.export
    @aeidon.deco.notify_frozen
    def open_main(self, path, encoding=None):
        """Read and parse subtitle data for main file from `path`.

        `encoding` can be ``None`` to use the system default encoding.
        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        Raise :exc:`aeidon.FormatError` if unable to detect format.
        Raise :exc:`aeidon.ParseError` if parsing fails.

        Return the amount of subtitles that needed to be moved in order to
        arrange them in ascending chronological order.
        """
        encoding = encoding or aeidon.util.get_default_encoding()
        # Check for a Unicode BOM first to avoid getting a FormatError in the
        # case where an unsuitable encoding decodes a file into garbage without
        # raising a UnicodeDecodeError. If a Unicode BOM is found, use the
        # corresponding Unicode encoding to open the file.
        bom_encoding = aeidon.encodings.detect_bom(path)
        if not bom_encoding in (encoding, None):
            return self.open_main(path, bom_encoding)
        format = aeidon.util.detect_format(path, encoding)
        self.main_file = aeidon.files.new(format, path, encoding)
        subtitles = self._read_file(self.main_file)
        self.subtitles, sort_count = self._sort_subtitles(subtitles)
        self.set_framerate(self.framerate, register=None)
        if self.main_file.format == aeidon.formats.MPSUB:
            # Get framerate from MPsub header.
            if self.main_file.framerate is not None:
                self.set_framerate(self.main_file.framerate, register=None)
        self.main_changed = 0
        # Deactivate possible translation file.
        self.tran_file = None
        self.tran_changed = None
        self.emit("main-file-opened", self.main_file)
        return sort_count

    def open_translation_require(self, path, encoding, align_method=None):
        assert self.main_file is not None
        assert aeidon.encodings.is_valid_code(encoding)

    def open_translation_ensure(self, *args, **kwargs):
        assert self.tran_file is not None
        assert self.tran_changed == 0

    @aeidon.deco.export
    @aeidon.deco.notify_frozen
    def open_translation(self, path, encoding=None, align_method=None):
        """Read and parse subtitle data for translation file from `path`.

        `encoding` can be ``None`` to use the system default encoding.

        `align_method` specifies how translation texts are attached to the
        existing subtitles. :attr:`aeidon.align_methods.NUMBER` is the simple
        way, which adds the translation texts in order, one-by-one to the
        exising subtitles. :attr:`aeidon.align_methods.POSITION` (the default)
        is the smarter way, which compares the position data in the translation
        subtitles with the existing subtitles, skips and inserts subtitles as
        needed to have at least a rough chronological match. The latter thus
        takes into account that not all subtitles are translated, or vice versa
        and that one main subtitle may correspond to two translation subtitles,
        or vice versa, as per length restrictions or whatever.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        Raise :exc:`aeidon.FormatError` if unable to detect format.
        Raise :exc:`aeidon.ParseError` if parsing fails.

        Return the amount of subtitles that needed to be moved in order to
        arrange them in ascending chronological order.
        """
        encoding = encoding or aeidon.util.get_default_encoding()
        align_method = align_method or aeidon.align_methods.POSITION
        # Check for a Unicode BOM first to avoid getting a FormatError in the
        # case where an unsuitable encoding decodes a file into garbage without
        # raising a UnicodeDecodeError. If a Unicode BOM is found, use the
        # corresponding Unicode encoding to open the file.
        bom_encoding = aeidon.encodings.detect_bom(path)
        if bom_encoding not in (encoding, None):
            return self.open_translation(path, bom_encoding, align_method)
        format = aeidon.util.detect_format(path, encoding)
        self.tran_file = aeidon.files.new(format, path, encoding)
        subtitles = self._read_file(self.tran_file)
        subtitles, sort_count = self._sort_subtitles(subtitles)
        for subtitle in subtitles:
            subtitle.framerate = self.framerate
        for subtitle in self.subtitles:
            subtitle.tran_text = ""
        blocked = self.block("subtitles-inserted")
        if align_method == aeidon.align_methods.POSITION:
            self._align_translations_by_position(subtitles)
        if align_method == aeidon.align_methods.NUMBER:
            self._align_translations_by_number(subtitles)
        self.unblock("subtitles-inserted", blocked)
        self.tran_changed = 0
        self.emit("translation-file-opened", self.tran_file)
        return sort_count



# Copyright (C) 2007-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re


class TestPattern(aeidon.TestCase):

    def setup_method(self, method):
        self.pattern = aeidon.Pattern()

    def test_get_flags(self):
        self.pattern.set_field("Flags", "DOTALL;UNICODE;")
        flags = self.pattern.get_flags()
        assert flags == re.DOTALL | re.UNICODE



# Copyright (C) 2006-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re


class TestLiner(aeidon.TestCase):

    def setup_method(self, method):
        self.liner = aeidon.Liner(re.compile(r"<.+?>"))
        self.liner.break_points.append((re.compile(r" (- )"), r"\n\1"))
        self.liner.break_points.append((re.compile(r"([.,?!]) "), r"\1\n"))

    def test__set_length_func(self):
        get_length = lambda x: len(x)**2 + 1
        self.liner.length_func = get_length
        assert self.liner.length_func == get_length
        assert self.liner._space_length == 2

    def test_break_lines__01(self):
        text = ("- Isn't he off on Saturdays? "
                "- Didn't he tell you?")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "- Isn't he off on Saturdays?\n"
            "- Didn't he tell you?")

    def test_break_lines__02(self):
        text = ("- Isn't he off on Saturdays? "
                "- He changed shifts. "
                "- Didn't he tell you? "
                "- Can you give him this when he next comes?")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "- Isn't he off on Saturdays?\n"
            "- He changed shifts. - Didn't he tell you?\n"
            "- Can you give him this when he next comes?")

    def test_break_lines__03(self):
        text = ("Isn't he off on Saturdays? "
                "He changed shifts.")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "Isn't he off on Saturdays?\n"
            "He changed shifts.")

    def test_break_lines__04(self):
        text = ("Isn't he off on Saturdays? "
                "He changed shifts. "
                "Didn't he tell you?")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "Isn't he off on Saturdays?\n"
            "He changed shifts. Didn't he tell you?")

    def test_break_lines__05(self):
        text = "Isn't he off on Saturdays"
        self.liner.set_text(text)
        assert self.liner.break_lines() == text

    def test_break_lines__06(self):
        text = ("Isn't he off on Saturdays "
                "He changed shifts "
                "Didn't he tell you?")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "Isn't he off on Saturdays He\n"
            "changed shifts Didn't he tell you?")

    def test_break_lines__07(self):
        text = ("Isn't he off on Saturdays "
                "He changed shifts "
                "Didn't he tell you "
                "Can you give him this when he next comes")
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "Isn't he off on Saturdays He changed\n"
            "shifts Didn't he tell you Can you\n"
            "give him this when he next comes")

    def test_break_lines__08(self):
        text = "test " * 50
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "test test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test test test")

    def test_break_lines__09(self):
        text = "test " * 60
        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test\n"
            "test test test test test test test test\n"
            "test test test test test test test\n"
            "test test test test test test test test")

    def test_break_lines__10(self):
        text = "testtesttesttest " * 101
        self.liner.set_text(text)
        assert self.liner.break_lines()

    def test_is_legal__long(self):
        text = "<i>I got to the restaurant a little early little early.</i>"
        self.liner.set_text(text)
        assert not self.liner.is_legal()

    def test_is_legal__ok(self):
        text = "<i>I got to the restaurant a little early.</i>"
        self.liner.set_text(text)
        assert self.liner.is_legal()

    def test_is_legal__unbreakable(self):
        text = "He'soffdutytodayHe'soffdutytodayHe'soffdutytoday."
        self.liner.set_text(text)
        assert self.liner.is_legal()

    def test_set_text(self):
        self.liner.set_text(" <i>foo</i> ")
        assert self.liner.text == "foo"



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Text markup for the MPL2 format."""

import aeidon

__all__ = ("MPL2",)


# pylint: disable=E1101,W0232

class MPL2(aeidon.tags.MicroDVD):

    """Text markup for the MPL2 format.

    MPL2 contains the following markup tags, all of which appear at the
    beginning of the line and affect up to the end of the line. In addition,
    any MicroDVD markup can be used.

     * ``\\...`` (bold)
     * ``/...`` (italic)
     * ``_...`` (underline)
    """

    format = aeidon.formats.MPL2

    def _main_decode_ensure(self, value, text):
        assert self.tag.search(value) is None

    def _main_decode(self, text):
        """Return `text` with decodable markup decoded."""
        text = self._decode_b(text, r"<\\>(.*?)</\\>", 1)
        text = self._decode_i(text, r"</>(.*?)<//>", 1)
        text = self._decode_u(text, r"<_>(.*?)</_>", 1)
        return aeidon.tags.MicroDVD._main_decode(self, text)

    def _pre_decode(self, text):
        """Return `text` with markup prepared for decoding."""
        text = self._pre_decode_identify(text)
        return aeidon.tags.MicroDVD._pre_decode(self, text)

    def _pre_decode_identify(self, text):
        """Return `text` with all tags identified and closed.

        ``\\``, ``/`` and ``_`` characters at the beginnings of lines are
        identified as tags and replaced with artificial tags ``<\\>``, ``</>``
        and ``<_>``. Closing tags are added to the ends of lines as artificial
        tags ``</\\>``, ``<//>`` and ``</_>``.
        """
        lines = text.split("\n")
        re_tag = self._get_regex(r"^([\\/_]+)(.*)$")
        for i, line in enumerate(lines):
            match = re_tag.search(line)
            if match is None: continue
            lines[i] = match.group(2)
            for tag in reversed(match.group(1)):
                lines[i] = "<%s>%s</%s>" % (tag, lines[i], tag)
        return "\n".join(lines)

    def _style_mpl2(self, text, tag, bounds=None):
        """Return `text` wrapped in markup `tag`."""
        a, z = bounds or (0, len(text))
        prefix = text[:a].split("\n")[-1]
        suffix = text[z:].split("\n")[0]
        re_alpha = self._get_regex(r"\w")
        # Return plain text if bounds does not define an entire line or
        # subtitle and thus cannot be marked without side-effects.
        if re_alpha.search(prefix): return text
        if re_alpha.search(suffix): return text
        styled_text = text[a:z].replace("\n", "\n%s" % tag)
        return "".join((text[:a], tag, styled_text, text[z:]))

    def bolden(self, text, bounds=None):
        """Return bolded `text`."""
        return self._style_mpl2(text, "\\", bounds)

    @property
    def italic_tag(self):
        """Regular expression for an italic markup tag."""
        return self._get_regex(r"(^/)|(\{[Yy]:i\})")

    def italicize(self, text, bounds=None):
        """Return italicized `text`."""
        return self._style_mpl2(text, "/", bounds)

    @property
    def tag(self):
        """Regular expression for any markup tag."""
        return self._get_regex(r"(^[\\/_]+)|(\{[CFSYcfsy]:.*?\})")

    def underline(self, text, bounds=None):
        """Return underlined `text`."""
        return self._style_mpl2(text, "_", bounds)



# Copyright (C) 2006-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs
import os
import sys
import time


class TestModule(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_affirm__false(self):
        self.raises(aeidon.AffirmationError,
                    aeidon.util.affirm,
                    1 == 0)

    def test_affirm__true(self):
        aeidon.util.affirm(0 == 0)

    def test_chardet_available(self):
        reload(aeidon.util)
        assert aeidon.util.chardet_available()

    def test_compare_versions(self):
        compare_versions = aeidon.util.compare_versions
        assert compare_versions("0.1.1", "0.1"  ) ==  1
        assert compare_versions("0.2"  , "0.1"  ) ==  1
        assert compare_versions("0.3"  , "0.3"  ) ==  0
        assert compare_versions("0.4"  , "0.4.1") == -1
        assert compare_versions("0.4"  , "0.5"  ) == -1

    def test_connect__private(self):
        # pylint: disable=W0201
        self._on_project_action_done = lambda *args: None
        aeidon.util.connect(self, "project", "action-done")

    def test_connect__public(self):
        # pylint: disable=W0201
        self.on_project_action_done = lambda *args: None
        aeidon.util.connect(self, "project", "action-done")

    def test_copy_dict(self):
        aeidon.util.copy_dict({1: 2, 3: {1: 2}})
        aeidon.util.copy_dict({1: 2, 3: [1, 2]})
        aeidon.util.copy_dict({1: 2, 3: set((1, 2))})

    def test_copy_list(self):
        aeidon.util.copy_list([1, 2, {1: 2}])
        aeidon.util.copy_list([1, 2, [1, 2]])
        aeidon.util.copy_list([1, 2, set((1, 2))])

    def test_detect_format(self):
        for format in aeidon.formats:
            path = self.new_temp_file(format)
            assert aeidon.util.detect_format(path, "ascii") == format

    def test_detect_format__format_error(self):
        path = self.new_subrip_file()
        aeidon.util.write(path, "xxx\n", "ascii")
        self.raises(aeidon.FormatError,
                    aeidon.util.detect_format, path, "ascii")

    def test_detect_newlines(self):
        aeidon.util.detect_newlines(self.new_subrip_file())

    def test_enchant_available(self):
        reload(aeidon.util)
        assert aeidon.util.enchant_available()

    def test_flatten(self):
        lst = [1, 2, [3, 4, [5, 6, [7]], 8], 9]
        lst = aeidon.util.flatten(lst)
        assert lst == [1, 2, 3, 4, 5, 6, 7, 8, 9]

    def test_get_all(self):
        assert aeidon.util.get_all(dir()) == ("self",)
        assert not aeidon.util.get_all(dir(), r"^[A-Z]")
        names = aeidon.util.get_all(dir(aeidon.util))
        assert len(names) < len(dir(aeidon.util))

    def test_get_chardet_version(self):
        reload(aeidon.util)
        assert aeidon.util.get_chardet_version()

    def test_get_default_encoding(self):
        assert aeidon.util.get_default_encoding()

    def test_get_default_newline(self):
        assert aeidon.util.get_default_newline()

    def test_get_enchant_version(self):
        reload(aeidon.util)
        assert aeidon.util.get_enchant_version()

    def test_get_encoding_alias(self):
        assert aeidon.util.get_encoding_alias("utf8") == "utf_8"
        assert aeidon.util.get_encoding_alias("johab") == "johab"

    def test_get_ranges__empty(self):
        assert aeidon.util.get_ranges([]) == []

    def test_get_ranges__non_empty(self):
        lst = [0, 0, 4, 5, 3, 7, 8, 2, 7]
        lst = aeidon.util.get_ranges(lst)
        assert lst == [[0], [2, 3, 4, 5], [7, 8]]

    def test_get_sorted_unique(self):
        lst = [4, 1, 5, 5, 1, 1, 3, 6, 4, 4]
        lst = aeidon.util.get_sorted_unique(lst)
        assert lst == [1, 3, 4, 5, 6]

    @aeidon.deco.monkey_patch(aeidon, "DATA_DIR")
    @aeidon.deco.monkey_patch(aeidon, "DATA_HOME_DIR")
    def test_get_template_header(self):
        for format in filter(lambda x: x.has_header, aeidon.formats):
            if format == aeidon.formats.MICRODVD: continue
            assert aeidon.util.get_template_header(format)
            dirs = aeidon.DATA_DIR, aeidon.DATA_HOME_DIR
            aeidon.DATA_HOME_DIR, aeidon.DATA_DIR = dirs
            assert aeidon.util.get_template_header(format)

    def test_get_unique__first(self):
        lst = [4, 1, 5, 5, 1, 1, 3, 6, 4, 4]
        lst = aeidon.util.get_unique(lst)
        assert lst == [4, 1, 5, 3, 6]

    def test_get_unique__last(self):
        lst = [4, 1, 5, 5, 1, 1, 3, 6, 4, 4]
        lst = aeidon.util.get_unique(lst, True)
        assert lst == [5, 1, 3, 6, 4]

    def test_is_command__false(self):
        assert not aeidon.util.is_command("???")

    def test_is_command__true(self):
        assert aeidon.util.is_command("echo")

    def test_last(self):
        assert aeidon.util.last(iter((1, 2, 3))) == 3

    def test_makedirs(self):
        path = "%s-%d" % (self.new_subrip_file(), time.time())
        aeidon.util.makedirs(path)
        aeidon.util.makedirs(path)
        os.rmdir(path)

    def test_normalize_newlines__mac(self):
        value = aeidon.util.normalize_newlines("test\rtest\r")
        assert value == "test\ntest\n"

    def test_normalize_newlines__unix(self):
        value = aeidon.util.normalize_newlines("test\ntest\n")
        assert value == "test\ntest\n"

    def test_normalize_newlines__windows(self):
        value = aeidon.util.normalize_newlines("test\r\ntest\r\n")
        assert value == "test\ntest\n"

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_path_to_uri__unix(self):
        sys.platform = "linux2"
        path = "/home/aeidon/a file.srt"
        uri = aeidon.util.path_to_uri(path)
        assert uri == "file:///home/aeidon/a%20file.srt"

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_path_to_uri__windows(self):
        sys.platform = "win32"
        path = "c:\\aeidon\\a file.srt"
        uri = aeidon.util.path_to_uri(path)
        assert uri == "file:///c%3A/aeidon/a%20file.srt"

    def test_print_read_io(self):
        try:
            open("/////", "r").read()
            raise AssertionError
        except IOError:
            aeidon.util.print_read_io(sys.exc_info(),
                                      "/////")

    def test_print_read_unicode(self):
        try:
            path = self.new_subrip_file()
            codecs.open(path, "r", "undefined").read()
            raise AssertionError
        except UnicodeError:
            aeidon.util.print_read_unicode(sys.exc_info(),
                                           path,
                                           "undefined")

    def test_print_remove_os(self):
        try:
            os.remove("/////")
        except OSError:
            aeidon.util.print_remove_os(sys.exc_info(),
                                        "/////")

    def test_print_write_io(self):
        try:
            open("/////", "w").write("\n")
            raise AssertionError
        except IOError:
            aeidon.util.print_write_io(sys.exc_info(),
                                       "/////")

    def test_print_write_unicode(self):
        try:
            path = self.new_subrip_file()
            codecs.open(path, "r", "ascii").write(u"\xc3\xb6\n")
            raise AssertionError
        except UnicodeError:
            aeidon.util.print_write_unicode(sys.exc_info(),
                                            path,
                                            "ascii")

    def test_read__basic(self):
        path = self.new_subrip_file()
        text = codecs.open(path, "r", "ascii").read().strip()
        assert aeidon.util.read(path) == text

    def test_read__fallback(self):
        path = self.new_subrip_file()
        codecs.open(path, "w", "utf_8").write(u"\xc3\xb6\n")
        assert aeidon.util.read(path, "ascii") == u"\xc3\xb6"

    def test_read__unicode_error(self):
        path = self.new_subrip_file()
        codecs.open(path, "w", "latin1").write(u"\xc3\xb6\n")
        self.raises(UnicodeError,
                    aeidon.util.read,
                    path, "ascii", None)

    def test_readlines__basic(self):
        path = self.new_subrip_file()
        lines = [x.rstrip() for x in open(path, "r").readlines()]
        assert aeidon.util.readlines(path) == lines

    def test_readlines__fallback(self):
        path = self.new_subrip_file()
        codecs.open(path, "w", "utf_8").write(u"\xc3\xb6\n")
        assert aeidon.util.readlines(path, "ascii") == [u"\xc3\xb6"]

    def test_readlines__unicode_error(self):
        path = self.new_subrip_file()
        codecs.open(path, "w", "latin1").write(u"\xc3\xb6\n")
        self.raises(UnicodeError,
                    aeidon.util.readlines,
                    path, "ascii", None)

    def test_replace_extension(self):
        format = aeidon.formats.SUBRIP
        value = aeidon.util.replace_extension("test.txt", format)
        assert value == "test.srt"

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_shell_quote__unix(self):
        sys.platform = "linux2"
        path = '/home/aeidon/a "file"\\.srt'
        path = aeidon.util.shell_quote(path)
        assert path == '"/home/aeidon/a \\"file\\"\\\\.srt"'

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_shell_quote__windows(self):
        sys.platform = "win32"
        path = 'c:\\aeidon\\a file.srt'
        path = aeidon.util.shell_quote(path)
        assert path == '"c:\\aeidon\\a file.srt"'

    def test_start_process(self):
        process = aeidon.util.start_process("echo")
        assert process.wait() == 0

    def test_title_to_lower_case(self):
        to_lower = aeidon.util.title_to_lower_case
        assert to_lower("Test") == "test"
        assert to_lower("TestCase") == "test_case"

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_uri_to_path__unix(self):
        sys.platform = "linux2"
        uri = "file:///home/aeidon/a%20file.srt"
        path = aeidon.util.uri_to_path(uri)
        assert path == "/home/aeidon/a file.srt"

    @aeidon.deco.monkey_patch(sys, "platform")
    def test_uri_to_path__windows(self):
        sys.platform = "win32"
        uri = "file:///c:/aeidon/a%20file.srt"
        path = aeidon.util.uri_to_path(uri)
        assert path == "c:\\aeidon\\a file.srt"

    def test_write__basic(self):
        text = "test\ntest\n"
        path = self.new_subrip_file()
        aeidon.util.write(path, text)
        fobj = codecs.open(path, "r", "ascii")
        assert fobj.read() == text

    def test_write__fallback(self):
        text = u"\xc3\xb6\n"
        path = self.new_subrip_file()
        aeidon.util.write(path, text, "ascii")
        fobj = codecs.open(path, "r", "utf_8")
        assert fobj.read() == text

    def test_write__unicode_error(self):
        path = self.new_subrip_file()
        self.raises(UnicodeError,
                    aeidon.util.write,
                    path, u"\xc3\xb6\n", "ascii", None)

    def test_writelines__basic(self):
        lines = ("test", "test")
        path = self.new_subrip_file()
        aeidon.util.writelines(path, lines)
        fobj = codecs.open(path, "r", "ascii")
        assert fobj.read() == "test\ntest\n"

    def test_writelines__fallback(self):
        text = u"\xc3\xb6"
        path = self.new_subrip_file()
        aeidon.util.writelines(path, (text,), "ascii")
        fobj = codecs.open(path, "r", "utf_8")
        assert fobj.read() == text + os.linesep

    def test_writelines__unicode_error(self):
        path = self.new_subrip_file()
        self.raises(UnicodeError,
                    aeidon.util.writelines,
                    path, ("\xc3\xb6\n",), "ascii", None)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Base class for dialogs for selecting subtitle files."""

import aeidon
import gaupol
import gtk
_ = aeidon.i18n._

__all__ = ("FileDialog",)


class FileDialog(gaupol.BuilderDialog):

    """Base class for dialogs for selecting subtitle files."""

    __metaclass__ = aeidon.Contractual
    _use_autodetection = False

    def _init_encoding_combo(self):
        """Initialize the character encoding combo box."""
        store = gtk.ListStore(str, str)
        self._encoding_combo.set_model(store)
        self._populate_encoding_combo()
        view = self._encoding_combo.get_child()
        view.set_displayed_row(0)
        renderer = gtk.CellRendererText()
        self._encoding_combo.pack_start(renderer, True)
        self._encoding_combo.add_attribute(renderer, "text", 1)
        function = gaupol.util.separate_combo
        self._encoding_combo.set_row_separator_func(function)

    def _init_filters(self):
        """Initialize file filters."""
        file_filter = gtk.FileFilter()
        file_filter.set_name(_("All files"))
        file_filter.add_pattern("*")
        self.add_filter(file_filter)
        file_filter = gtk.FileFilter()
        file_filter.set_name(_("All supported files"))
        for format in aeidon.formats:
            pattern = "*."
            for x in format.extension[1:]:
                pattern += "[%s%s]" % (x.upper(), x.lower())
            file_filter.add_pattern(pattern)
        self.add_filter(file_filter)
        self.set_filter(file_filter)
        for format in aeidon.formats:
            extension = format.extension
            pattern = "*."
            for x in extension[1:]:
                pattern += "[%s%s]" % (x.upper(), x.lower())
            format = format.label
            name = _("%(format)s (*%(extension)s)") % locals()
            file_filter = gtk.FileFilter()
            file_filter.set_name(name)
            file_filter.add_pattern(pattern)
            self.add_filter(file_filter)

    def _on_encoding_combo_changed(self, *args):
        """Show the encoding selection dialog."""
        encoding = self.get_encoding()
        if encoding != "other": return
        dialog = gaupol.MenuEncodingDialog(self._dialog)
        response = gaupol.util.run_dialog(dialog)
        encoding = dialog.get_encoding()
        visible = dialog.get_visible_encodings()
        dialog.destroy()
        self._encoding_combo.set_active(0)
        if response != gtk.RESPONSE_OK: return
        gaupol.conf.encoding.visible = visible
        self._populate_encoding_combo(encoding)
        self.set_encoding(encoding)

    def _populate_encoding_combo(self, custom=None):
        """Populate the encoding combo box, including custom encoding."""
        encodings = list(gaupol.conf.encoding.visible)
        locale = aeidon.encodings.get_locale_code()
        encodings.insert(0, locale)
        encodings.append(custom)
        while None in encodings:
            encodings.remove(None)
        encodings = aeidon.util.get_unique(encodings)
        encodings = encodings or ["utf_8"]
        for i, encoding in enumerate(encodings):
            name = aeidon.encodings.code_to_long_name(encoding)
            encodings[i] = (encoding, name)
        if locale is not None:
            name = aeidon.encodings.get_locale_long_name()
            encodings[0] = (locale, name)
        a = (0 if locale is None else 1)
        encodings[a:] = sorted(encodings[a:], key=lambda x: x[1])
        separator = gaupol.COMBO_SEPARATOR
        if self._use_autodetection:
            encodings.append((separator, separator))
            encodings.append(("auto", _("Auto-detected")))
        encodings.append((separator, separator))
        encodings.append(("other", _("Other\342\200\246")))
        self._encoding_combo.get_model().clear()
        store = self._encoding_combo.get_model()
        for encoding in encodings:
            store.append(tuple(encoding))

    def get_encoding_ensure(self, value):
        if not value in ("auto", "other", None):
            assert aeidon.encodings.is_valid_code(value)

    def get_encoding(self):
        """Return the selected encoding or ``None``."""
        index = self._encoding_combo.get_active()
        if index < 0: return None
        store = self._encoding_combo.get_model()
        return store[index][0]

    def set_encoding(self, encoding):
        """Set the selected encoding."""
        if encoding is None: return
        store = self._encoding_combo.get_model()
        for i in range(len(store)):
            if store[i][0] == encoding:
                return self._encoding_combo.set_active(i)
        if aeidon.encodings.is_valid_code(encoding):
            ## Add encoding if not found in store.
            self._populate_encoding_combo(encoding)
            return self.set_encoding(encoding)
        self._encoding_combo.set_active(0)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import random
_ = aeidon.i18n._


class TestPage(gaupol.TestCase):

    def setup_method(self, method):
        mode = random.choice((aeidon.modes.TIME, aeidon.modes.FRAME))
        gaupol.conf.editor.mode = mode
        self.page = self.new_page()

    def test__on_conf_editor_notify_use_undo_limit(self):
        gaupol.conf.editor.use_undo_limit = True
        gaupol.conf.editor.undo_limit = 1
        gaupol.conf.editor.use_undo_limit = False

    def test__on_conf_editor_notify_undo_limit(self):
        gaupol.conf.editor.use_undo_limit = True
        gaupol.conf.editor.undo_limit = 1
        gaupol.conf.editor.undo_limit = 100

    def test__on_project_main_file_opened(self):
        path = self.new_subrip_file()
        self.page.project.open_main(path, "ascii")

    def test__on_project_main_texts_changed(self):
        doc = aeidon.documents.MAIN
        self.page.project.set_text(0, doc, "test")

    def test__on_project_positions_changed(self):
        self.page.project.shift_positions(None, 3.0)

    def test__on_project_subtitles_changed(self):
        rows = range(len(self.page.project.subtitles))
        self.page._on_project_subtitles_changed(self.page.project, rows)

    def test__on_project_subtitles_inserted(self):
        self.page.project.insert_subtitles((0, 1))

    def test__on_project_subtitles_removed(self):
        self.page.project.remove_subtitles((2, 3))

    def test__on_project_subtitles_removed__51(self):
        self.page.project.insert_subtitles(range(0, 51))
        self.page.project.remove_subtitles(range(0, 51))

    def test__on_project_translation_file_opened(self):
        path = self.new_subrip_file()
        self.page.project.open_translation(path, "ascii")

    def test__on_project_translation_texts_changed(self):
        doc = aeidon.documents.TRAN
        self.page.project.set_text(0, doc, "test")

    def test_document_to_text_column__main(self):
        doc = aeidon.documents.MAIN
        col = self.page.document_to_text_column(doc)
        assert col == self.page.view.columns.MAIN_TEXT

    def test_document_to_text_column__translation(self):
        doc = aeidon.documents.TRAN
        col = self.page.document_to_text_column(doc)
        assert col == self.page.view.columns.TRAN_TEXT

    def test_document_to_text_column__value_error(self):
        self.raises(ValueError, self.page.document_to_text_column, None)

    def test_get_main_basename(self):
        self.page.project.main_file.path = "/tmp/root.srt"
        name = self.page.get_main_basename()
        assert name == "root.srt"

    def test_get_main_basename__untitle(self):
        self.page.project.main_file = None
        name = self.page.get_main_basename()
        assert name == self.page.untitle

    def test_get_translation_basename(self):
        self.page.project.tran_file.path = "/tmp/root.sub"
        name = self.page.get_translation_basename()
        assert name == "root.sub"

    def test_get_translation_basename__translation(self):
        self.page.project.main_file.path = "/tmp/root.srt"
        self.page.project.tran_file = None
        name = self.page.get_translation_basename()
        assert name == _("%s translation") % "root"

    def test_get_translation_basename__untitle_translation(self):
        self.page.project.main_file = None
        self.page.project.tran_file = None
        name = self.page.get_translation_basename()
        assert name == _("%s translation") % self.page.untitle

    def test_reload_view(self):
        rows = range(len(self.page.project.subtitles))
        fields = [x for x in gaupol.fields]
        fields.remove(gaupol.fields.NUMBER)
        self.page.reload_view(rows, fields)

    def test_reload_view_all(self):
        self.page.reload_view_all()

    def test_text_column_to_document__main(self):
        col = self.page.view.columns.MAIN_TEXT
        doc = self.page.text_column_to_document(col)
        assert doc == aeidon.documents.MAIN

    def test_text_column_to_document__translation(self):
        col = self.page.view.columns.TRAN_TEXT
        doc = self.page.text_column_to_document(col)
        assert doc == aeidon.documents.TRAN

    def test_text_column_to_document__value_error(self):
        self.raises(ValueError, self.page.text_column_to_document, None)

    def test_update_tab_label(self):
        title = self.page.update_tab_label()
        assert title == self.page.get_main_basename()

    def test_update_tab_label__changed(self):
        self.page.project.remove_subtitles([1])
        title = self.page.update_tab_label()
        assert title == "*%s" % self.page.get_main_basename()



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Enumerations for video player application types."""

import aeidon
import os
import sys

__all__ = ("players",)


def _get_mplayer_executable():
    if sys.platform == "win32":
        directory = os.environ.get("PROGRAMFILES", "C:\\Program Files")
        path = os.path.join(directory, "MPlayer", "mplayer.exe")
        return aeidon.util.shell_quote(path)
    return "mplayer"

def _get_vlc_executable():
    if sys.platform == "win32":
        directory = os.environ.get("PROGRAMFILES", "C:\\Program Files")
        path = os.path.join(directory, "VideoLAN", "VLC", "vlc.exe")
        return aeidon.util.shell_quote(path)
    return "vlc"


class MPlayer(aeidon.EnumerationItem):

    command = " ".join((_get_mplayer_executable(),
                        "-identify",
                        "-osdlevel 2",
                        "-ss $SECONDS",
                        "-slang",
                        "-noautosub",
                        "-sub $SUBFILE",
                        "$VIDEOFILE",))

    if sys.platform != "win32":
        # Required for mplayer to work if gaupol was started
        # as a background process (&) from a terminal window.
        # http://www.mplayerhq.hu/DOCS/HTML/en/faq.html#idp11051520
        command = "%s < /dev/null" % command
    command_utf_8 = " ".join((_get_mplayer_executable(),
                              "-identify",
                              "-osdlevel 2",
                              "-ss $SECONDS",
                              "-slang",
                              "-noautosub",
                              "-sub $SUBFILE",
                              "-utf8",
                              "$VIDEOFILE",))

    if sys.platform != "win32":
        # Required for mplayer to work if gaupol was started
        # as a background process (&) from a terminal window.
        # http://www.mplayerhq.hu/DOCS/HTML/en/faq.html#idp11051520
        command_utf_8 = "%s < /dev/null" % command_utf_8
    label = "MPlayer"


class VLC(aeidon.EnumerationItem):

    command = " ".join((_get_vlc_executable(),
                        "$VIDEOFILE",
                        ":start-time=$SECONDS",
                        ":sub-file=$SUBFILE",))

    command_utf_8 = " ".join((_get_vlc_executable(),
                              "$VIDEOFILE",
                              ":start-time=$SECONDS",
                              ":sub-file=$SUBFILE",
                              ":subsdec-encoding=UTF-8",))

    label = "VLC"


players = aeidon.Enumeration()
players.MPLAYER = MPlayer()
players.VLC = VLC()



# Copyright (C) 2006-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Breaker of text into lines according to preferred break points."""

from __future__ import division

import aeidon
import copy
import math
import re
import sys

__all__ = ("Liner",)


class Liner(aeidon.Parser):

    """Breaker of text into lines according to preferred break points.

    :class:`Liner` operates by first joining all lines by spaces and then
    trying different breaks. Breaks are tried first from :attr:`break_points`
    in the order they are defined and if that fails, lines are split at word
    boundaries. Lines are attempted to be broken so that the variance of line
    lengths is minimized, while satisfying legality constraints, of which
    :attr:`max_length` is never broken and :attr:`max_lines` is broken if
    absolutely necessary. Variance minimizing splits are sought using a brute
    force algorithm as long as the text is not abnormally long.
    :attr:`max_deviation` can be used to control the threshold for discarding
    legal solutions for subtitles with three or more lines and a variance too
    great for the subtitle to be typeset elegantly.

    :ivar _space_length: Length of a space according to :attr:`length_func`
    :ivar break_points: List of tuples of regex object, replacement
    :ivar length_func: A function that returns the length of its argument
    :ivar max_deviation: Maximum deviation for texts with three or more lines
    :ivar max_length: Maximum length of a line in units of :attr:`_length_func`
    :ivar max_lines: Maximum preferred amount of lines (may be exceeded)
    """

    __metaclass__ = aeidon.Contractual
    _re_multi_space = re.compile(r" {2,}")

    def __init__(self, re_tag=None, clean_func=None):
        """Initialize a :class:`Liner` object.

        `re_tag` should be a regular expression object.
        """
        aeidon.Parser.__init__(self, re_tag, clean_func)
        self._length_func = len
        self._space_length = 1
        self.break_points = []
        self.max_deviation = 0.16
        self.max_length = 44
        self.max_lines = 2

    def _break_on_pattern(self, max_lines, regex, replacement):
        """Break the text into lines based on `regex`.

        Return ``True`` if breaks made and result is elegant and legal.
        """
        # Prefer elegance over compactness by not using
        # a maximum line count less than the preferred maximum.
        max_lines = max(max_lines, self.max_lines)
        self.text = regex.sub(replacement, self.text).strip()
        if 0 < self.text.count("\n") < max_lines:
            if self.is_legal() and (not self._is_deviant()):
                # Success with one item per line.
                return True
        if self.text.count("\n") > 0:
            self._join_even(max_lines)
            if self.is_legal() and (not self._is_deviant()):
                # Success with multiple items spread evenly on lines.
                return True
        self.text = self.text.replace("\n", " ")
        return False

    def _break_on_words(self, max_lines):
        """Break the text into lines based on words."""
        self.text = self.text.replace(" ", "\n")
        self.text = self.text.replace("\n-\n", "\n- ")
        self._join_even(max_lines)
        if self.is_legal(): return True
        self.text = self.text.replace("\n", " ")
        return False

    def _get_length(self, lengths):
        """Return the length of items joined by spaces."""
        return (sum(lengths) + (len(lengths) - 1) * self._space_length)

    def _get_break_ensure(self, value, lengths):
        assert 0 <= value <= len(lengths)

    def _get_break(self, lengths):
        """Return index of break in two with minimum length difference.

        Index is brute forced within reason and the result is optimal.
        """
        min_index = 0
        min_diff = sys.maxint
        start = self._get_start_index(lengths, 2)
        for i in range(start, len(lengths)):
            a = self._get_length(lengths[:i])
            b = self._get_length(lengths[i:])
            diff = abs(a - b)
            if diff < min_diff:
                min_index = i
                min_diff = diff
            # End if already past halfway.
            if b < a: break
        return min_index

    def _get_breaks_ensure(self, value, lengths, max_lines):
        for index in value:
            assert 0 <= index <= len(lengths)

    def _get_breaks(self, lengths, max_lines):
        """Return indices of breaks with minimum length variance.

        Indexes are brute forced within reason and the result is optimal if
        ``max_lines`` is less than six, otherwise result is (potentially) ugly.
        """
        if max_lines == 1:
            return []
        if max_lines == 2:
            return [self._get_break(lengths)]
        if (max_lines > 5) or (sum(lengths) > (5 * self.max_length)):
            # Avoid slow brute forcing by combining items.
            return self._get_breaks_ugly(lengths, max_lines)
        return self._get_breaks_pretty(lengths, max_lines)

    def _get_breaks_pretty_ensure(self, value, lengths, max_lines):
        for index in value:
            assert 0 <= index <= len(lengths)

    def _get_breaks_pretty(self, lengths, max_lines):
        """Return indices of breaks with minimum length variance.

        Indexes are brute forced within reason and the result is optimal.
        """
        min_indices = []
        min_squares = sys.maxint
        start = self._get_start_index(lengths, max_lines)
        for i in range(start, len(lengths) - max_lines + 2):
            indices = self._get_breaks(lengths[i:], max_lines - 1)
            indices = [i] + [x + i for x in indices]
            borders = [0] + indices + [len(lengths)]
            line_lengths = []
            for j in range(1, len(borders)):
                a, z = borders[j - 1:j + 1]
                line_lengths.append(self._get_length(lengths[a:z]))
            mean = sum(line_lengths) / max_lines
            squares = sum([(x - mean)**2 for x in line_lengths])
            if round(min_squares - squares, 6) > 0:
                min_indices = indices
                min_squares = squares
            a = self._get_length(lengths[:i])
            b = self._get_length(lengths[i:])
            # End if remaining less than average line length.
            if a > (b / (max_lines - 1)): break
        return min_indices

    def _get_breaks_ugly_ensure(self, value, lengths, max_lines):
        for index in value:
            assert 0 <= index <= len(lengths)

    def _get_breaks_ugly(self, lengths, max_lines):
        """Return indices of breaks with minimum length variance.

        Items are first broken into half of ``max_lines`` and then each these
        lines is further broken internally into two. The result is not optimal,
        but is obtained fast.
        """
         # Splits can only be made to an even number of lines.
        if (max_lines % 2 != 0):
            max_lines = max_lines - 1
        indices = self._get_breaks(lengths, int(max_lines / 2))
        borders = [0] + indices + [len(lengths)]
        for i in range(1, len(borders)):
            a, z = borders[i - 1:i + 1]
            indices.append(self._get_break(lengths[a:z]) + a)
        return sorted(indices)

    def _get_start_index_ensure(self, value, lengths, max_lines):
        assert 0 <= value <= len(lengths)

    def _get_start_index(self, lengths, max_lines):
        """Return the index for the first break candidate for items.

        The start index is determined based on the line length mean with the
        purpose to avoid brute forcing insanely small indices.
        """
        if len(lengths) < 3:
            return 1
        mean = sum(lengths) / max_lines
        for i in range(2, len(lengths)):
            a = self._get_length(lengths[:i])
            if a > mean:
                return i - 1
        return 1

    def _invariant(self):
        assert isinstance(self._length_func(""), (int, float))

    def _is_deviant(self):
        """Return ``True`` if line lengths deviate too much."""
        line_count = self.text.count("\n") + 1
        if line_count <= 2: return
        lengths = [self._length_func(x) for x in self.text.split("\n")]
        mean = sum(lengths) / line_count
        std = math.sqrt(sum([(x - mean)**2 for x in lengths]) / line_count)
        return ((std / self.max_length) > self.max_deviation)

    def _join_even_ensure(self, value, max_lines):
        assert self.text.count("\n") + 1 <= max_lines

    def _join_even(self, max_lines):
        """Join lines evenly so that ``max_lines`` is not violated."""
        text = ""
        items = self.text.split("\n")
        lengths = [self._length_func(x) for x in items]
        indices = self._get_breaks(lengths, max_lines)
        for i in range(len(items)):
            prefix = ("\n" if i in indices else " ")
            text = text + prefix + items[i]
        self.text = text.strip()

    def break_lines(self):
        """Break lines and return text."""
        self.text = self.text.replace("\n", " ")
        self.pattern = self._re_multi_space
        self.replacement = " "
        self.replace_all()
        text = self.text
        tags = copy.deepcopy(self._tags)
        for max_lines in range(1, 50):
            # Try preferred break points one by one.
            for regex, replacement in self.break_points:
                self.text = text
                self._tags = copy.deepcopy(tags)
                args = (max_lines, regex, replacement)
                if self._break_on_pattern(*args):
                    return self.get_text()
            # Fall back to breaking by words.
            self.text = text
            self._tags = copy.deepcopy(tags)
            if self._break_on_words(max_lines):
                return self.get_text()
        return self.get_text()

    def is_legal(self):
        """Return ``True`` if the text does not violate :attr:`max_length`."""
        for line in self.text.split("\n"):
            length = self._length_func(line)
            if (" " in line) and (length > self.max_length):
                return False
        return True

    @property
    def length_func(self):
        """Return the length function used."""
        return self._length_func

    @length_func.setter
    def length_func(self, func):
        """Set the length function to use."""
        self._length_func = func
        self._space_length = func(" ")

    def set_text(self, text, next=True):
        """Set the target text to search in and parse it.

        `next` should be ``True`` to start at beginning, ``False`` for end.
        """
        aeidon.Parser.set_text(self, text.strip(), next)
        self.text = self.text.strip()



# Copyright (C) 2005-2007,2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestPositionAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_adjust_durations__gap(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end = "00:00:02.000"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end = "00:00:03.000"
        subtitles[2].start = "00:00:04.000"
        self.project.adjust_durations(self.project.get_all_indices(),
                                      gap=0.3)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end == "00:00:01.700"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end == "00:00:03.000"
        assert subtitles[2].start == "00:00:04.000"

    def test_adjust_durations__lengthen(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end = "00:00:01.100"
        subtitles[0].main_text = "1234567890"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end = "00:00:02.100"
        subtitles[1].main_text = "12345678901234567890"
        self.project.adjust_durations(None,
                                      speed=20,
                                      lengthen=True)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end == "00:00:03.000"

    def test_adjust_durations__maximum(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end = "00:00:01.100"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end = "00:00:02.100"
        self.project.adjust_durations(None,
                                      speed=10,
                                      lengthen=True,
                                      maximum=0.5)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end == "00:00:02.500"

    def test_adjust_durations__minimum(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end = "00:00:01.900"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end = "00:00:02.900"
        self.project.adjust_durations(None,
                                      speed=1000,
                                      shorten=True,
                                      minimum=0.5)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end == "00:00:02.500"

    def test_adjust_durations__shorten(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end = "00:00:01.900"
        subtitles[0].main_text = "1234567890"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end = "00:00:02.900"
        subtitles[1].main_text = "12345678901234567890"
        self.project.adjust_durations(None,
                                      speed=100,
                                      shorten=True)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end == "00:00:01.100"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end == "00:00:02.200"

    def test_convert_framerate__frame(self):
        self.project.open_main(self.new_microdvd_file(), "ascii")
        self.project.subtitles[0].start = 100
        self.project.subtitles[1].start = 200
        self.project.convert_framerate(self.project.get_all_indices(),
                                       aeidon.framerates.FPS_23_976,
                                       aeidon.framerates.FPS_25_000)

        assert self.project.framerate == aeidon.framerates.FPS_25_000
        for subtitle in self.project.subtitles:
            assert subtitle.framerate == aeidon.framerates.FPS_25_000
        assert self.project.subtitles[0].start == 104
        assert self.project.subtitles[1].start == 209

    def test_convert_framerate__time(self):
        self.project.open_main(self.new_subrip_file(), "ascii")
        self.project.subtitles[0].start = "00:00:01.000"
        self.project.subtitles[1].start = "00:00:02.000"
        self.project.convert_framerate(None,
                                       aeidon.framerates.FPS_23_976,
                                       aeidon.framerates.FPS_25_000)

        assert self.project.framerate == aeidon.framerates.FPS_25_000
        for subtitle in self.project.subtitles:
            assert subtitle.framerate == aeidon.framerates.FPS_25_000
        assert self.project.subtitles[0].start == "00:00:00.959"
        assert self.project.subtitles[1].start == "00:00:01.918"

    def test_set_framerate(self):
        self.project.set_framerate(aeidon.framerates.FPS_25_000)
        assert self.project.framerate == aeidon.framerates.FPS_25_000
        for subtitle in self.project.subtitles:
            assert subtitle.framerate == aeidon.framerates.FPS_25_000

    @aeidon.deco.reversion_test
    def test_shift_positions__frame(self):
        orig_subtitles = [x.copy() for x in self.project.subtitles]
        indices = self.project.get_all_indices()
        self.project.shift_positions(indices, -10)
        for i, subtitle in enumerate(self.project.subtitles):
            start = orig_subtitles[i].start_frame - 10
            assert subtitle.start_frame == start
            end = orig_subtitles[i].end_frame - 10
            assert subtitle.end_frame == end

    @aeidon.deco.reversion_test
    def test_shift_positions__time(self):
        orig_subtitles = [x.copy() for x in self.project.subtitles]
        self.project.shift_positions(None, "00:00:01.000")
        for i, subtitle in enumerate(self.project.subtitles):
            start = round(orig_subtitles[i].start_seconds + 1.0, 3)
            assert round(subtitle.start_seconds, 3) == start
            end = round(orig_subtitles[i].end_seconds + 1.0, 3)
            assert round(subtitle.end_seconds, 3) == end

    @aeidon.deco.reversion_test
    def test_shift_positions__seconds(self):
        orig_subtitles = [x.copy() for x in self.project.subtitles]
        self.project.shift_positions(None, 1.0)
        for i, subtitle in enumerate(self.project.subtitles):
            start = round(orig_subtitles[i].start_seconds + 1.0, 3)
            assert round(subtitle.start_seconds, 3) == start
            end = round(orig_subtitles[i].end_seconds + 1.0, 3)
            assert round(subtitle.end_seconds, 3) == end

    @aeidon.deco.reversion_test
    def test_transform_positions__frame(self):
        indices = self.project.get_all_indices()
        self.project.transform_positions(indices, (2, 10), (6, 100))
        assert self.project.subtitles[2].start_frame == 10
        for subtitle in self.project.subtitles[3:6]:
            assert 10 < subtitle.start_frame < 100
        assert self.project.subtitles[6].start_frame == 100

    @aeidon.deco.reversion_test
    def test_transform_positions__seconds(self):
        self.project.transform_positions(None, (2, 20.0), (6, 200.0))
        assert self.project.subtitles[2].start_seconds == 20.0
        for subtitle in self.project.subtitles[3:6]:
            assert 20.0 < subtitle.start_seconds < 200.0
        assert self.project.subtitles[6].start_seconds == 200.0

    @aeidon.deco.reversion_test
    def test_transform_positions__time(self):
        a, b = "00:00:01.000", "00:00:45.000"
        self.project.transform_positions(None, (2, a), (6, b))
        assert self.project.subtitles[2].start_time == a
        for subtitle in self.project.subtitles[3:6]:
            assert a < subtitle.start_time < b
        assert self.project.subtitles[6].start_time == b



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Updating the application GUI."""

import aeidon
import gaupol
import glib
import gtk
import os
_ = aeidon.i18n._


class UpdateAgent(aeidon.Delegate):

    """Updating the application GUI.

    :ivar _message_id: A :class:`gtk.Statusbar` message ID
    :ivar _message_tag: A timeout from :func:`glib.timeout_add`
    """

    __metaclass__ = aeidon.Contractual

    def __init__(self, master):
        """Initialize an :class:`UpdateAgent` object."""
        aeidon.Delegate.__init__(self, master)
        self._message_id = None
        self._message_tag = None

    def _disable_widgets(self):
        """Make widgets insensitive and blank."""
        self.window.set_title("Gaupol")
        self.video_button.get_data("label").set_text("")
        self.push_message(None)

    @aeidon.deco.export
    def _on_activate_next_project_activate(self, *args):
        """Activate the project in the next tab."""
        self.notebook.next_page()

    @aeidon.deco.export
    def _on_activate_previous_project_activate(self, *args):
        """Activate the project in the previous tab."""
        self.notebook.prev_page()

    @aeidon.deco.export
    def _on_conf_application_window_notify_toolbar_style(self, *args):
        """Change the style of the main toolbar."""
        toolbar = self.uim.get_widget("/ui/main_toolbar")
        style = gaupol.conf.application_window.toolbar_style
        if style == gaupol.toolbar_styles.DEFAULT:
            return toolbar.unset_style()
        toolbar.set_style(style.value)

    @aeidon.deco.export
    def _on_move_tab_left_activate(self, *args):
        """Move the current tab to the left."""
        page = self.get_current_page()
        scroller = page.view.get_parent()
        index = self.pages.index(page)
        self.notebook.reorder_child(scroller, index - 1)

    @aeidon.deco.export
    def _on_move_tab_right_activate(self, *args):
        """Move the current tab to the right."""
        page = self.get_current_page()
        scroller = page.view.get_parent()
        index = self.pages.index(page)
        self.notebook.reorder_child(scroller, index + 1)

    def _on_notebook_page_reordered_ensure(self, value, *args, **kwargs):
        for i, page in enumerate(self.pages):
            ith_page = self.notebook.get_nth_page(i)
            assert ith_page.get_child() == page.view

    @aeidon.deco.export
    def _on_notebook_page_reordered(self, notebook, scroller, index):
        """Update the list of pages to match the new order."""
        view = scroller.get_child()
        page = filter(lambda x: x.view is view, self.pages)[0]
        self.pages.remove(page)
        self.pages.insert(index, page)
        self.update_gui()
        self.emit("pages-reordered", page, index)

    @aeidon.deco.export
    def _on_notebook_switch_page(self, notebook, pointer, index):
        """Update GUI for the page switched to."""
        if not self.pages: return
        self.update_gui()
        page = self.pages[index]
        page.view.grab_focus()
        self.emit("page-switched", page)

    @aeidon.deco.export
    def _on_view_button_press_event(self, view, event):
        """Display a right-click pop-up menu to edit data."""
        if event.button != 3: return
        x = int(event.x)
        y = int(event.y)
        value = view.get_path_at_pos(x, y)
        if value is None: return
        path, column, x, y = value
        if path[0] not in view.get_selected_rows():
            view.set_cursor(path[0], column)
            view.update_headers()
        menu = self.uim.get_widget("/ui/view_popup")
        menu.popup(None, None, None, event.button, event.time)
        return True

    @aeidon.deco.export
    def _on_view_move_cursor(self, *args):
        """Update GUI after moving cursor in the view."""
        self.update_gui()

    @aeidon.deco.export
    def _on_view_selection_changed(self, *args):
        """Update GUI after changing selection in the view."""
        self.update_gui()

    @aeidon.deco.export
    def _on_window_window_state_event(self, window, event):
        """Save window maximization."""
        state = event.new_window_state
        maximized = bool(state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
        gaupol.conf.application_window.maximized = maximized

    def _update_actions(self, page):
        """Update sensitivities of all actions for page."""
        for name in ("main-safe", "main-unsafe"):
            action_group = self.get_action_group(name)
            for action in action_group.list_actions():
                action.update_sensitivity(self, page)

    def _update_revert(self, page):
        """Update tooltips for undo and redo actions."""
        if page is None: return
        if page.project.can_undo():
            action = self.get_action("undo_action")
            action.props.tooltip = _('Undo "%s"') % (
                page.project.undoables[0].description)
        if page.project.can_redo():
            action = self.get_action("redo_action")
            action.props.tooltip = _('Redo "%s"') % (
                page.project.redoables[0].description)

    def _update_widgets(self, page):
        """Update states of all widgets for `page`."""
        if page is None:
            return self._disable_widgets()
        self.window.set_title(page.tab_label.get_text())
        self.get_mode_action(page.edit_mode).set_active(True)
        self.get_framerate_action(page.project.framerate).set_active(True)
        self.framerate_combo.set_active(page.project.framerate)
        for field in gaupol.fields:
            col = getattr(page.view.columns, field.name)
            visible = page.view.get_column(col).props.visible
            self.get_column_action(field).set_active(visible)
        video = os.path.basename(page.project.video_path or "")
        self.video_button.get_data("label").set_text(video)
        self.video_button.set_tooltip_text(video or None)

    @aeidon.deco.export
    def flash_message(self, message):
        """Show `message` in statusbar for a short while."""
        self.push_message(message)
        self._message_tag = gaupol.util.delay_add(6000,
                                                  self.push_message,
                                                  None)

    @aeidon.deco.export
    def push_message(self, message):
        """Show `message` in the statusbar."""
        if self._message_tag is not None:
            glib.source_remove(self._message_tag)
        if self._message_id is not None:
            self.statusbar.remove_message(0, self._message_id)
        event_box = self.statusbar.get_ancestor(gtk.EventBox)
        self.statusbar.set_tooltip_text(message)
        if message is not None:
            self._message_id = self.statusbar.push(0, message)

    @aeidon.deco.export
    def update_gui(self):
        """Update widget sensitivities and states for the current page."""
        page = self.get_current_page()
        self._update_actions(page)
        self._update_widgets(page)
        self._update_revert(page)
        self.extension_manager.update_extensions(page)



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol
import gtk


class TestHeaderDialog(gaupol.TestCase):

    def run__dialog__both(self):
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__main(self):
        self.dialog.destroy()
        self.setup_main()
        self.dialog.run()
        self.dialog.destroy()

    def run__dialog__translation(self):
        self.dialog.destroy()
        self.setup_translation()
        self.dialog.run()
        self.dialog.destroy()

    def run__show_mpsub_error_dialog(self):
        self.dialog._show_mpsub_error_dialog()

    def setup_both(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        page = self.application.get_current_page()
        path = self.new_temp_file(aeidon.formats.SUBVIEWER2)
        self.application.open_main(path, "ascii")
        path = self.new_temp_file(aeidon.formats.MPSUB)
        self.application.open_translation(path, "ascii")
        self.dialog = gaupol.HeaderDialog(self.application.window,
                                          self.application)

        self.dialog.show()

    def setup_main(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        page = self.application.get_current_page()
        path = self.new_temp_file(aeidon.formats.SUBVIEWER2)
        self.application.open_main(path, "ascii")
        self.dialog = gaupol.HeaderDialog(self.application.window,
                                          self.application)

        self.dialog.show()

    def setup_method(self, method):
        return self.setup_both()

    def setup_translation(self):
        # pylint: disable=W0201
        self.application = self.new_application()
        page = self.application.get_current_page()
        path = self.new_temp_file(aeidon.formats.SUBRIP)
        self.application.open_main(path, "ascii")
        path = self.new_temp_file(aeidon.formats.SUBVIEWER2)
        self.application.open_translation(path, "ascii")
        self.dialog = gaupol.HeaderDialog(self.application.window,
                                          self.application)

        self.dialog.show()

    def test__on_copy_down_button_clicked(self):
        self.dialog._copy_down_button.clicked()
        value = self.dialog._get_translation_header()
        assert value == self.dialog._main_file.header

    def test__on_copy_up_button_clicked(self):
        self.dialog._copy_up_button.clicked()
        value = self.dialog._get_main_header()
        assert value == self.dialog._tran_file.header

    def test__on_main_clear_button_clicked(self):
        self.dialog._main_clear_button.clicked()
        value = self.dialog._get_main_header()
        assert value == ""

    def test__on_main_template_button_clicked(self):
        self.dialog._main_template_button.clicked()
        value = self.dialog._get_main_header()
        format = self.dialog._main_file.format
        assert value == aeidon.util.get_template_header(format)

    def test__on_main_revert_button_clicked(self):
        self.dialog._main_clear_button.clicked()
        self.dialog._main_revert_button.clicked()
        value = self.dialog._get_main_header()
        assert value == self.dialog._main_file.header

    def test__on_response(self):
        self.dialog.response(gtk.RESPONSE_OK)

    def test__on_tran_clear_button_clicked(self):
        self.dialog._tran_clear_button.clicked()
        value = self.dialog._get_translation_header()
        assert value == ""

    def test__on_tran_template_button_clicked(self):
        self.dialog._tran_template_button.clicked()
        value = self.dialog._get_translation_header()
        format = self.dialog._tran_file.format
        assert value == aeidon.util.get_template_header(format)

    def test__on_tran_revert_button_clicked(self):
        self.dialog._tran_clear_button.clicked()
        self.dialog._tran_revert_button.clicked()
        value = self.dialog._get_translation_header()
        assert value == self.dialog._tran_file.header

    @aeidon.deco.monkey_patch(gaupol.util, "flash_dialog")
    def test___show_mpsub_error_dialog(self):
        gaupol.util.flash_dialog = lambda *args: gtk.RESPONSE_OK
        self.dialog._show_mpsub_error_dialog()



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Miscellaneous helper methods."""

import aeidon
import gaupol
import gtk


class UtilityAgent(aeidon.Delegate):

    """Miscellaneous helper methods."""

    __metaclass__ = aeidon.Contractual

    def get_action_ensure(self, value, name):
        assert value is not None

    @aeidon.deco.export
    def get_action(self, name):
        """Return action from UI manager by `name`."""
        for action_group in self.uim.get_action_groups():
            action = action_group.get_action(name)
            if action is not None: return action
        raise ValueError("Action %s not found" % repr(name))

    @aeidon.deco.export
    def get_action_group(self, name):
        """Return action group from UI manager by `name`."""
        groups = self.uim.get_action_groups()
        return [x for x in groups if x.get_name() == name][0]

    @aeidon.deco.export
    def get_column_action(self, field):
        """Return action from UI manager to hide or show column."""
        name = gaupol.field_actions[field]
        return self.get_action(name)

    @aeidon.deco.export
    def get_current_page(self):
        """Return the currently active page or ``None``."""
        index = self.notebook.get_current_page()
        if index < 0: return None
        return self.pages[index]

    @aeidon.deco.export
    def get_framerate_action(self, framerate):
        """Return action from UI manager to select framerate."""
        name = gaupol.framerate_actions[framerate]
        return self.get_action(name)

    @aeidon.deco.export
    def get_menu_item(self, name):
        """Return menu item from UI manager by `name`."""
        widgets = self.get_action(name).get_proxies()
        return [x for x in widgets if isinstance(x, gtk.MenuItem)][0]

    @aeidon.deco.export
    def get_mode_action(self, mode):
        """Return action from UI manager to select mode."""
        name = gaupol.mode_actions[mode]
        return self.get_action(name)

    @aeidon.deco.export
    def get_target_pages(self, target):
        """Return a sequence of pages corresponding to `target`."""
        if target == gaupol.targets.SELECTED:
            return (self.get_current_page(),)
        if target == gaupol.targets.CURRENT:
            return (self.get_current_page(),)
        if target == gaupol.targets.ALL:
            return tuple(self.pages)
        raise ValueError("Invalid target: %s" % repr(target))

    @aeidon.deco.export
    def get_target_rows(self, target):
        """Return rows corresponding to `target` or ``None`` for all."""
        if target != gaupol.targets.SELECTED: return None
        page = self.get_current_page()
        return page.view.get_selected_rows()

    @aeidon.deco.export
    def get_tool_item(self, name):
        """Return tool item from UI manager by `name`."""
        widgets = self.get_action(name).get_proxies()
        return [x for x in widgets if isinstance(x, gtk.ToolItem)][0]

    def set_current_page_require(self, page):
        assert page in self.pages

    @aeidon.deco.export
    def set_current_page(self, page):
        """Set the currently active page."""
        index = self.pages.index(page)
        self.notebook.set_current_page(index)



# Copyright (C) 2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Separate object that can be activated and deactivated during runtime."""

__all__ = ("Extension",)


class Extension(object):

    """Separate object that can be activated and deactivated during runtime."""

    def setup(self, application):
        """Setup extension for use with `application`.

        This method is called every time the extension is associated with
        `application`, i.e. both when it is manually activated and also every
        time right after `application` start.
        """
        pass

    def show_help(self):
        """Show documentation on using extension.

        Subclasses can override this to, for example, launch a web browser with
        :func:`gaupol.util.show_uri` to view HTML documentation.
        """
        raise NotImplementedError

    def show_preferences_dialog(self, parent):
        """Show a preferences dialog for configuring extension.

        `parent` is the parent window that the dialog can be centered on.
        """
        raise NotImplementedError

    def teardown(self, application):
        """End use of extension with `application`.

        This method is called every time the extension is unassociated with
        `application`, i.e. both when it is manually deactivated and also every
        time right before `application` exit.
        """
        pass

    def update(self, application, page):
        """Update state of extension for `application` and active `page`.

        The main purpose of this method is to update sensitivities of UI
        manager actions associated with this extension.
        """
        pass



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import gaupol
import gtk


class _TestFileDialog(gaupol.TestCase):

    # pylint: disable=E1101

    def run__dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def test__on_encoding_combo_changed(self):
        responder = iter((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
        def run_dialog(dialog):
            selection = dialog._tree_view.get_selection()
            selection.select_path(0)
            return responder.next()
        gaupol.util.run_dialog = run_dialog
        store = self.dialog._encoding_combo.get_model()
        self.dialog._encoding_combo.set_active(len(store) - 1)
        self.dialog._encoding_combo.set_active(len(store) - 1)

    def test_get_encoding(self):
        encoding = gaupol.conf.encoding.visible[0]
        self.dialog.set_encoding(encoding)
        assert self.dialog.get_encoding() == encoding
        self.dialog.set_encoding("johab")
        assert self.dialog.get_encoding() == "johab"

    def test_set_encoding(self):
        encoding = gaupol.conf.encoding.visible[0]
        self.dialog.set_encoding(encoding)
        assert self.dialog.get_encoding() == encoding
        self.dialog.set_encoding("johab")
        assert self.dialog.get_encoding() == "johab"



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class PuppetObservable(aeidon.Observable):

    signals = ("do",)

    def __init__(self):
        aeidon.Observable.__init__(self)
        self.x = 0


class TestObservable(aeidon.TestCase):

    def on_notify_x(self, obj, value):
        assert obj is self.obs
        assert value == self.obs.x
        self.notify_count += 1

    def on_do(self, obj):
        assert obj is self.obs
        self.do_count += 1

    def setup_method(self, method):
        self.obs = PuppetObservable()
        self.do_count = 0
        self.notify_count = 0
        self.obs.connect("do", self.on_do)
        self.obs.connect("notify::x", self.on_notify_x)

    def test__add_signal(self):
        self.obs._add_signal("undo")
        self.obs.emit("undo")

    def test__validate__immutable(self):
        for value in ([], {}, set()):
            assert self.obs._validate("", value) == value
            assert self.obs._validate("", value) is not value

    def test__validate__mutable(self):
        for value in (True, "", 1, 1.0, (), frozenset()):
            assert self.obs._validate("", value) == value
            assert self.obs._validate("", value) is value

    def test_block(self):
        assert self.obs.block("do")
        assert not self.obs.block("do")
        self.obs.emit("do")
        assert self.do_count == 0

    def test_block_all(self):
        assert self.obs.block_all()
        assert not self.obs.block_all()
        self.obs.emit("do")
        assert self.do_count == 0
        self.obs.notify("x")
        assert self.notify_count == 0

    def test_connect(self):
        self.obs.connect("do", lambda *args: None)
        self.obs.connect("notify::x", lambda *args: None)

    def test_disconnect(self):
        self.obs.disconnect("do", self.on_do)
        self.obs.emit("do")
        assert self.do_count == 0

    def test_emit(self):
        self.obs.x = 1
        assert self.notify_count == 1
        self.obs.emit("do")
        assert self.do_count == 1

    def test_freeze_notify(self):
        assert self.obs.freeze_notify()
        assert not self.obs.freeze_notify()
        self.obs.x = 1
        assert self.notify_count == 0

    def test_notify(self):
        self.obs.notify("x")
        assert self.notify_count == 1

    def test_thaw_notify(self):
        self.obs.freeze_notify()
        self.obs.x = 1
        self.obs.x = 2
        assert self.obs.thaw_notify()
        assert not self.obs.thaw_notify()
        assert self.notify_count == 1

    def test_unblock(self):
        self.obs.block("do")
        assert self.obs.unblock("do")
        assert not self.obs.unblock("do")
        self.obs.emit("do")
        assert self.do_count == 1

    def test_unblock_all(self):
        self.obs.block_all()
        assert self.obs.unblock_all()
        assert not self.obs.unblock_all()
        self.obs.emit("do")
        assert self.do_count == 1
        self.obs.notify("x")
        assert self.notify_count == 1



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""SubRip file."""

import aeidon
import re

__all__ = ("SubRip",)


class SubRip(aeidon.SubtitleFile):

    """SubRip file."""

    _re_time_line = re.compile((
            # Techically all these fields should have fixed widths, but in the
            # name of being liberal in accepting input, accept lesser widths
            # assuming that they are just lacking zero-padding from the side
            # that is further from the decimal point.
            r"^(-?\d{1,2}:\d{1,2}:\d{1,2},\d{1,3}) -->"
            r" (-?\d{1,2}:\d{1,2}:\d{1,2},\d{1,3})"
            r"(  X1:(\d+) X2:(\d+) Y1:(\d+) Y2:(\d+))?\s*$"))

    format = aeidon.formats.SUBRIP
    mode = aeidon.modes.TIME

    def _read_lines(self):
        """Read file to a unicoded list of lines."""
        lines = ["\n"]
        for line in aeidon.SubtitleFile._read_lines(self):
            lines.append(line)
            match = self._re_time_line.match(line)
            if match is None: continue
            # Remove numbers and blank lines above them.
            if lines[-2].strip().isdigit():
                if not lines[-3].strip(): lines.pop(-3)
                lines.pop(-2)
        return lines

    def read(self):
        """Read file and return subtitles.

        Raise :exc:`IOError` if reading fails.
        Raise :exc:`UnicodeError` if decoding fails.
        """
        subtitles = []
        for line in self._read_lines():
            match = self._re_time_line.match(line)
            if match is None:
                if subtitles[-1].main_text:
                    subtitles[-1].main_text += "\n"
                subtitles[-1].main_text += line
                continue
            subtitle = self._get_subtitle()
            subtitle.start = subtitle.calc.parse_time(match.group(1))
            subtitle.end = subtitle.calc.parse_time(match.group(2))
            if match.group(3) is not None:
                subtitle.subrip.x1 = int(match.group(4))
                subtitle.subrip.x2 = int(match.group(5))
                subtitle.subrip.y1 = int(match.group(6))
                subtitle.subrip.y2 = int(match.group(7))
            subtitles.append(subtitle)
        return subtitles

    def write_to_file(self, subtitles, doc, fobj):
        """Write `subtitles` from `doc` to `fobj`.

        Raise :exc:`IOError` if writing fails.
        Raise :exc:`UnicodeError` if encoding fails.
        """
        n = self.newline.value
        for i, subtitle in enumerate(subtitles):
            if i > 0: fobj.write(n)
            fobj.write("%d%s" % ((i + 1), n))
            start = subtitle.start_time.replace(".", ",")
            end = subtitle.end_time.replace(".", ",")
            fobj.write("%s --> %s" % (start, end))
            # Write Extended SubRip coordinates only if the container
            # has been initialized and the coordinates make some sense.
            if subtitle.has_container("subrip"):
                x1 = subtitle.subrip.x1
                x2 = subtitle.subrip.x2
                y1 = subtitle.subrip.y1
                y2 = subtitle.subrip.y2
                if not (x1 == x2 == y1 == y2 == 0):
                    fobj.write("  X1:%03d X2:%03d" % (x1, x2))
                    fobj.write( " Y1:%03d Y2:%03d" % (y1, y2))
            text = subtitle.get_text(doc).replace("\n", n)
            fobj.write("%s%s%s" % (n, text, n))



# Copyright (C) 2005-2008,2010 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Closing pages and quitting Gaupol."""

import aeidon
import gaupol
import gtk
import os
_ = aeidon.i18n._


class CloseAgent(aeidon.Delegate):

    """Closing pages and quitting Gaupol."""

    def _confirm_close(self, page):
        """Close `page` after asking to save its documents.

        Raise :exc:`gaupol.Default` if cancelled and `page` not closed.
        """
        docs = self._need_confirmation(page)
        if len(docs) == 2:
            return self._confirm_close_multiple((page,))
        if aeidon.documents.MAIN in docs:
            return self._confirm_close_main(page)
        if aeidon.documents.TRAN in docs:
            return self._confirm_close_translation(page)

    def _confirm_close_main(self, page):
        """Close `page` after asking to save its main document.

        Raise :exc:`gaupol.Default` if cancelled and page not closed.
        """
        title = _('Save changes to main document "%s" before closing?')
        title = title % page.get_main_basename()
        message = _("If you don't save, changes will be permanently lost.")
        dialog = gaupol.WarningDialog(self.window, title, message)
        dialog.add_button(_("Close _Without Saving"), gtk.RESPONSE_NO)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_YES)
        response = gaupol.util.flash_dialog(dialog)
        if not response in (gtk.RESPONSE_YES, gtk.RESPONSE_NO):
            raise gaupol.Default
        if response == gtk.RESPONSE_YES:
            self.save_main(page)

    def _confirm_close_multiple(self, pages):
        """Close `pages` after asking to save their documents.

        Raise :exc:`gaupol.Default` if cancelled and `pages` not closed.
        """
        dialog = gaupol.MultiCloseDialog(self.window, self, pages)
        response = gaupol.util.flash_dialog(dialog)
        if not response in (gtk.RESPONSE_YES, gtk.RESPONSE_NO):
            raise gaupol.Default

    def _confirm_close_translation(self, page):
        """Close `page` after asking to save its translation document.

        Raise :exc:`gaupol.Default` if cancelled and page not closed.
        """
        title = _('Save changes to translation document "%s" before closing?')
        title = title % page.get_translation_basename()
        message = _("If you don't save, changes will be permanently lost.")
        dialog = gaupol.WarningDialog(self.window, title, message)
        dialog.add_button(_("Close _Without Saving"), gtk.RESPONSE_NO)
        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        dialog.add_button(gtk.STOCK_SAVE, gtk.RESPONSE_YES)
        dialog.set_default_response(gtk.RESPONSE_YES)
        response = gaupol.util.flash_dialog(dialog)
        if not response in (gtk.RESPONSE_YES, gtk.RESPONSE_NO):
            raise gaupol.Default
        if response == gtk.RESPONSE_YES:
            self.save_translation(page)

    def _need_confirmation(self, page):
        """Return documents in `page` with unsaved changes."""
        docs = []
        if page.project.main_changed:
            docs.append(aeidon.documents.MAIN)
        elif page.project.main_file is not None:
            if not os.path.isfile(page.project.main_file.path):
                docs.append(aeidon.documents.MAIN)
        if page.project.tran_changed:
            docs.append(aeidon.documents.TRAN)
        elif page.project.tran_file is not None:
            if not os.path.isfile(page.project.tran_file.path):
                docs.append(aeidon.documents.TRAN)
        return tuple(docs)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_close_all_projects_activate(self, *args):
        """Close all open projects."""
        self.close_all()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_close_project_activate(self, *args):
        """Close project."""
        self.close(self.get_current_page())

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_page_close_request(self, page, *args):
        """Close project."""
        self.close(page)

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_quit_activate(self, *args):
        """Quit Gaupol."""
        self.quit()

    @aeidon.deco.export
    @aeidon.deco.silent(gaupol.Default)
    def _on_window_delete_event(self, *args):
        """Quit Gaupol."""
        self.quit()
        return True

    def _save_window_geometry(self):
        """Save the geometry of the application and output windows."""
        if not gaupol.conf.application_window.maximized:
            conf = gaupol.conf.application_window
            conf.size = list(self.window.get_size())
            conf.position = list(self.window.get_position())
        if not gaupol.conf.output_window.maximized:
            conf = gaupol.conf.output_window
            conf.size = list(self.output_window.get_size())
            conf.position = list(self.output_window.get_position())

    @aeidon.deco.export
    def close_all(self):
        """Close all pages after asking to save their documents.

        Raise :exc:`gaupol.Default` if cancelled and all pages not closed.
        """
        if sum((len(self._need_confirmation(x)) for x in self.pages)) > 1:
            return self._confirm_close_multiple(tuple(self.pages))
        while self.pages:
            self.close(self.pages[-1])

    @aeidon.deco.export
    def close(self, page, confirm=True):
        """Close `page` after asking to save its documents.

        If `confirm` is ``False`` do not ask to save documents.
        Raise :exc:`gaupol.Default` if cancelled and page was not closed.
        """
        if confirm:
            self._confirm_close(page)
        if page not in self.pages: return
        index = self.pages.index(page)
        if self.notebook.get_current_page() == index:
            self.notebook.next_page()
        self.notebook.remove_page(index)
        self.pages.remove(page)
        self.update_gui()
        self.emit("page-closed", page)

    @aeidon.deco.export
    def quit(self):
        """Quit Gaupol.

        Raise :exc:`gaupol.Default` if cancelled.
        """
        self.emit("quit")
        self.close_all()
        self.extension_manager.teardown_extensions()
        self._save_window_geometry()
        try: gtk.main_quit()
        except RuntimeError:
            raise SystemExit(1)



# Copyright (C) 2008-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

"""Containers for additional format-specific subtitle attributes.

Instances of containers must be able to gracefully survive a
:func:`copy.deepcopy` operation. If the container is in some way complicated,
it should implement :meth:`__copy__` and :meth:`__deepcopy__` methods to ensure
this.
"""


class SubRip(object):

    """Subtitle box pixel coordinates for extended SubRip format.

    :ivar x1: Subtitle corner X coordinate in pixels
    :ivar y1: Subtitle corner Y coordinate in pixels
    :ivar x2: Subtitle corner X coordinate in pixels
    :ivar y2: Subtitle corner Y coordinate in pixels
    """

    x1 = y1 = x2 = y2 = 0


class SubStationAlpha(object):

    """Attributes for all versions of Sub Station Alpha formats.

    :ivar marked: 0 for not marked or 1 for marked (default 0)
    :ivar layer: Layer used by collusion detection (default 0)
    :ivar style: Style name (default "Default")
    :ivar name: Name of the character speaking (default "")
    :ivar margin_l: Left margin override (default 0, i.e. default margins)
    :ivar margin_r: Right margin override (default 0, i.e. default margins)
    :ivar margin_v: Bottom margin override (default 0, i.e. default margins)
    :ivar effect: Transition effect (default "", i.e. no effect)
    """

    marked = 0
    layer = 0
    style = "Default"
    name = ""
    margin_l = 0
    margin_r = 0
    margin_v = 0
    effect = ""


def new(name):
    """Return a new container instance given the container's `name`."""
    if name == "ssa":
        return SubStationAlpha()
    if name == "subrip":
        return SubRip()
    raise ValueError("Invalid name: %s" % repr(name))



# Copyright (C) 2005-2009 Osmo Salomaa
#
# This file is part of Gaupol.
#
# Gaupol is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# Gaupol is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# Gaupol. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSetAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    @aeidon.deco.reversion_test
    def test_set_duration__frame(self):
        subtitles = self.project.subtitles
        self.project.set_duration(0, 1)
        assert subtitles[0].duration_frame == 1

    @aeidon.deco.reversion_test
    def test_set_duration__seconds(self):
        subtitles = self.project.subtitles
        self.project.set_duration(0, 100.0)
        assert subtitles[0].duration_seconds == 100.0

    @aeidon.deco.reversion_test
    def test_set_duration__time(self):
        subtitles = self.project.subtitles
        self.project.set_duration(0, "00:01:11.111")
        assert subtitles[0].duration_time == "00:01:11.111"

    @aeidon.deco.reversion_test
    def test_set_end__frame(self):
        subtitles = self.project.subtitles
        self.project.set_end(0, 600000)
        assert subtitles[0].end_frame == 600000

    @aeidon.deco.reversion_test
    def test_set_end__seconds(self):
        subtitles = self.project.subtitles
        self.project.set_end(0, 500.0)
        assert subtitles[0].end_seconds == 500.0

    @aeidon.deco.reversion_test
    def test_set_end__time(self):
        subtitles = self.project.subtitles
        self.project.set_end(0, "00:22:00.000")
        assert subtitles[0].end_time == "00:22:00.000"

    def test_set_main_text(self):
        self.project.subtitles[0].main_text = "a"
        self.project.set_main_text(0, "m")

    @aeidon.deco.reversion_test
    def test_set_start__frame(self):
        subtitles = self.project.subtitles
        self.project.set_start(0, -1000)
        assert subtitles[0].start_frame == -1000

    @aeidon.deco.reversion_test
    def test_set_start__reorder(self):
        subtitles = self.project.subtitles
        text_0 = subtitles[0].main_text
        text_3 = subtitles[3].main_text
        self.project.set_start(3, -1000)
        assert subtitles[0].start_frame == -1000
        assert subtitles[0].main_text == text_3
        assert subtitles[1].main_text == text_0

    @aeidon.deco.reversion_test
    def test_set_start__seconds(self):
        subtitles = self.project.subtitles
        self.project.set_start(0, -100.0)
        assert subtitles[0].start_seconds == -100.0

    @aeidon.deco.reversion_test
    def test_set_start__time(self):
        subtitles = self.project.subtitles
        self.project.set_start(0, "-00:01:00.000")
        assert subtitles[0].start_time == "-00:01:00.000"

    @aeidon.deco.reversion_test
    def test_set_text__main(self):
        self.project.set_text(0, aeidon.documents.MAIN, "m")
        assert self.project.subtitles[0].main_text == "m"

    @aeidon.deco.reversion_test
    def test_set_text__translation(self):
        self.project.set_text(0, aeidon.documents.TRAN, "t")
        assert self.project.subtitles[0].tran_text == "t"

    def test_set_translation_text(self):
        self.project.set_translation_text(0, "t")
        assert self.project.subtitles[0].tran_text == "t"
